#!/usr/bin/env python
# -*- coding: UTF-8 -*-
import logging

from django.db import transaction
from django_restql.mixins import QueryArgumentsMixin
from rest_framework.decorators import action
from rest_framework.permissions import IsAuthenticated
from rest_framework.utils.serializer_helpers import ReturnList
from rest_framework.viewsets import ModelViewSet

from django_base_ai.system.ext_field.models_safe import MaskField, MaskType
from django_base_ai.utils.filters import DataLevelPermissionsFilter
from django_base_ai.utils.import_export_mixin import ExportSerializerMixin, ImportSerializerMixin
from django_base_ai.utils.json_response import DetailResponse, ErrorResponse, SuccessResponse
from django_base_ai.utils.permission import CustomPermission

logger = logging.getLogger(__name__)


class CustomModelViewSet(ModelViewSet, ImportSerializerMixin, ExportSerializerMixin, QueryArgumentsMixin):
    """
    自定义的ModelViewSet:
    统一标准的返回格式;新增,查询,修改可使用不同序列化器
    (1)ORM性能优化, 尽可能使用values_queryset形式
    (2)xxx_serializer_class 某个方法下使用的序列化器(xxx=create|update|list|retrieve|destroy)
    (3)filter_fields = '__all__' 默认支持全部model中的字段查询(除json字段外) 精确查询的字段
    (4)import_field_dict={} 导入时的字段字典 {model值: model的label}
    (5)export_field_label = [] 导出时的字段
    """

    values_queryset = None
    ordering_fields = "__all__"
    create_serializer_class = None
    update_serializer_class = None
    filter_fields = "__all__"
    search_fields = ()  # 模糊查询的字段
    extra_filter_backends = [DataLevelPermissionsFilter]
    permission_classes = [CustomPermission]
    import_field_dict = {}
    export_field_label = {}

    def filter_queryset(self, queryset):
        for backend in set(set(self.filter_backends) | set(self.extra_filter_backends or [])):
            queryset = backend().filter_queryset(self.request, queryset, self)
        return queryset

    def set_mask_fields(self, serializer):
        # 掩码
        if serializer is None or len(serializer.data) == 0:
            return []
        serializer_data = serializer.data

        def mask_field(item):
            for k, v in self.serializer_class.Meta.mask_fields.items():
                if isinstance(v, MaskType):
                    item[k] = MaskField(value=item.get(k, ""), mask_type=v).get_value()
                else:
                    item[k] = MaskField(
                        value=item.get(k, ""),
                        mask_type=v.get("type", MaskType.DEFAULT),
                        first_value=v.get("first_value", 0),
                        last_value=v.get("last_value", 0),
                    ).get_value()

        if hasattr(self.serializer_class.Meta, "mask_fields") and len(self.serializer_class.Meta.mask_fields) > 0:
            if isinstance(serializer_data, ReturnList):
                for item in serializer.data:
                    mask_field(item)
            else:
                mask_field(serializer_data)
        return serializer_data

    def get_queryset(self):
        if getattr(self, "values_queryset", None):
            return self.values_queryset
        return super().get_queryset()

    def get_serializer_class(self):
        action_serializer_name = f"{self.action}_serializer_class"
        action_serializer_class = getattr(self, action_serializer_name, None)
        if action_serializer_class:
            return action_serializer_class
        return super().get_serializer_class()

    # 通过many=True直接改造原有的API，使其可以批量创建
    def get_serializer(self, *args, **kwargs):
        serializer_class = self.get_serializer_class()
        kwargs.setdefault("context", self.get_serializer_context())
        if isinstance(self.request.data, list):
            with transaction.atomic():
                return serializer_class(many=True, *args, **kwargs)
        else:
            return serializer_class(*args, **kwargs)

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data, request=request)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
        return DetailResponse(data=serializer.data, msg="新增成功")

    def list(self, request, *args, **kwargs):
        queryset = self.filter_queryset(self.get_queryset())
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True, request=request)
            # 掩码
            serializer_data = self.set_mask_fields(serializer)
            return self.get_paginated_response(serializer_data)
        serializer = self.get_serializer(queryset, many=True, request=request)
        return SuccessResponse(data=serializer.data, msg="获取成功")

    def retrieve(self, request, *args, **kwargs):
        instance = self.get_object()
        serializer = self.get_serializer(instance)
        serializer_data = self.set_mask_fields(serializer)
        return DetailResponse(data=serializer_data, msg="获取成功")

    def update(self, request, *args, **kwargs):
        partial = request.GET.get("partial", False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=request.data, request=request, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)

        if getattr(instance, "_prefetched_objects_cache", None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}
        return DetailResponse(data=serializer.data, msg="更新成功")

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()
        instance.delete()
        return DetailResponse(data=[], msg="删除成功")

    # 数据移交-仅支持单层移交
    @action(methods=["post"], detail=False, permission_classes=[IsAuthenticated])
    def transfer(self, request, *args, **kwargs):
        nid = request.data.get("nid", 0)
        uid = request.data.get("uid", 0)
        if nid is None or nid == 0 or uid is None or uid == 0:
            return ErrorResponse(msg="用户不存在,请检查参数")
        from django.contrib.auth import get_user_model

        user_model = get_user_model()
        new_user = user_model.objects.filter(id=uid).first()
        if new_user is None:
            return ErrorResponse(msg="用户不存在,请检查参数")
        instance = self.queryset.filter(id=nid, creator=request.user.id).first()
        if instance is None:
            return ErrorResponse(msg="数据不存在,请检查参数")
        instance.creator = new_user.id
        instance.modifier = new_user.id
        instance.save()
        serializer = self.get_serializer(instance)
        return DetailResponse(data=serializer.data, msg="获取成功")

    # 多选删除
    @action(methods=["delete"], detail=False)
    def multiple_delete(self, request, *args, **kwargs):
        request_data = request.data
        keys = request_data.get("keys", None)
        if keys is None:
            return ErrorResponse(msg="参数不合法,请稍后再试")
        self.get_queryset().filter(id__in=keys).delete()
        return SuccessResponse(data=[], msg="删除成功")



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
from django.db.backends.mysql import base, features
from django.utils.functional import cached_property


class DatabaseFeatures(features.DatabaseFeatures):
    @cached_property
    def is_sql_auto_is_null_enabled(self):
        with self.connection.cursor() as cursor:
            cursor.execute("SELECT @@SQL_AUTO_IS_NULL")
            result = cursor.fetchone()
            return result and result["@@SQL_AUTO_IS_NULL"] == 1


class DatabaseWrapper(base.DatabaseWrapper):
    features_class = DatabaseFeatures

    def create_cursor(self, name=None):
        cursor = self.connection.cursor(self.Database.cursors.DictCursor)
        return base.CursorWrapper(cursor)

    @cached_property
    def mysql_server_info(self):
        with self.temporary_connection() as cursor:
            cursor.execute("SELECT VERSION()")
            return cursor.fetchone()["VERSION()"]



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import time
import uuid
from datetime import date, timedelta

from django.apps import apps
from django.core.exceptions import ObjectDoesNotExist
from django.conf import settings
from django.db import ProgrammingError, connection, models, transaction

# 数据库名称
app_label_db_name = "django_base_ai"
# 表前缀（使用主项目 settings 中的 TABLE_PREFIX）
table_prefix = getattr(settings, "TABLE_PREFIX", "")


class SoftDeleteQuerySet(models.query.QuerySet):
    @transaction.atomic
    def delete(self, cascade=True):
        if cascade:  # delete one by one if cascade
            for obj in self.all():
                obj.delete(cascade=cascade)
        return self.update(is_deleted=True)

    def hard_delete(self):
        return super().delete()


class SoftDeleteManager(models.Manager):
    """支持软删除"""

    def __init__(self, *args, **kwargs):
        self.__add_is_del_filter = False
        super().__init__(*args, **kwargs)

    def filter(self, *args, **kwargs):
        # 考虑是否主动传入is_deleted
        if kwargs.get("is_deleted") is not None:
            self.__add_is_del_filter = kwargs.get("is_deleted")
        return super().filter(*args, **kwargs)

    def get_queryset(self):
        if self.__add_is_del_filter:
            return SoftDeleteQuerySet(self.model, using=self._db).exclude(is_deleted=False)
        return SoftDeleteQuerySet(self.model).exclude(is_deleted=True)

    def get_by_natural_key(self, name):
        return SoftDeleteQuerySet(self.model).get(username=name)


# 自动生成GUID
def newuuid():
    return f"{str(uuid.uuid4())}_{time.time_ns()}"


def get_month_range(start_day, end_day):
    months = (end_day.year - start_day.year) * 12 + end_day.month - start_day.month
    month_range = [
        f"{start_day.year + mon // 12}-{str(mon % 12 + 1).zfill(2)}-01"
        for mon in range(start_day.month - 1, start_day.month + months)
    ]
    return month_range


class SoftDeleteModel(models.Model):
    """
    软删除模型
    一旦继承,就将开启软删除
    """

    is_deleted = models.BooleanField(verbose_name="是否软删除", help_text="是否软删除", default=False, db_index=True)
    objects = SoftDeleteManager()

    class Meta:
        abstract = True
        verbose_name = "软删除模型"
        verbose_name_plural = verbose_name

    @transaction.atomic
    def delete(self, using=None, cascade=True, *args, **kwargs):
        """
        重写删除方法,直接开启软删除
        """
        self.is_deleted = True
        self.save(using=using)
        if cascade:
            self.delete_related_objects(raise_exception=True)
        # raise Exception("delete_related_objects")

    def hard_delete(self):
        return super().delete()

    soft_delete_kwargs = {
        "related_names": [],
    }

    @classmethod
    def _get_kwargs(cls):
        return cls.soft_delete_kwargs

    @classmethod
    def _get_relations(cls):
        relations = {"foreign": [], "self": []}
        related_fields = cls._get_kwargs().get("related_names", [])
        if not related_fields:
            fields = cls._meta.get_fields(include_hidden=True)
            mutated_fields = [field for field in fields if field.is_relation and hasattr(field, "related_name")]
            m2m_models = [field.through for field in mutated_fields if field.many_to_many]
            related_fields = [
                field.related_name
                for field in mutated_fields
                if not field.many_to_many and field.related_model not in m2m_models and field.related_name
            ]
            tree_model_field = [
                field.field.name
                for field in mutated_fields
                if not field.many_to_many and field.related_model is field.model
            ]
            relations["self"] = f"{tree_model_field[0]}_id" if len(tree_model_field) == 1 else None
        relations["foreign"] = related_fields
        return relations

    def _is_cascade(self, relation):
        on_delete_case = self._meta.get_field(relation).on_delete.__name__
        return on_delete_case == "CASCADE"

    def _get_related_objects(self, relation):
        qs = getattr(self, relation)
        if isinstance(qs, models.Manager):
            return qs
        return

    def related_objects(self, raise_exception=False, use_soft_manager=False):
        relations = self._get_relations()
        objects = {}
        for relation in relations["foreign"]:
            try:
                qs = self._get_related_objects(relation)
            except ObjectDoesNotExist as e:
                if raise_exception:
                    raise e
                continue
            else:
                objects[relation] = qs
        if relations["self"]:
            objects["self"] = self.__class__.objects.filter(**{relations["self"]: self.id})
        print(f"related_objects: {objects}", flush=True)
        return objects

    def delete_related_objects(self, raise_exception=False):
        for relation, qs in self.related_objects(raise_exception=raise_exception).items():
            if relation == "self":
                qs.delete()
                continue
            if self._is_cascade(relation):
                print(f"model {self.__class__} : cascade delete {relation} objects {qs.all()}", flush=True)
                qs.all().delete()
            else:
                print(f"model {self.__class__} : protect delete {relation} objects {qs.all()}", flush=True)
                if qs.all().exists():
                    self.hard_delete()
                qs.all().hard_delete()
        # raise Exception("xxxxxxxxxxx for test xxxxxxxxxxx")


class CoreModel(models.Model):
    """
    核心标准抽象模型模型,可直接继承使用
    增加审计字段, 覆盖字段时, 字段名称请勿修改, 必须统一审计字段名称
    """

    id = models.BigAutoField(
        primary_key=True,
        help_text="主键",
        verbose_name="Id",
    )
    creator = models.CharField(
        max_length=255, null=True, blank=True, db_index=True, help_text="创建人", verbose_name="创建人"
    )
    modifier = models.CharField(
        max_length=255, null=True, blank=True, db_index=True, help_text="修改人", verbose_name="修改人"
    )
    dept_belong_id = models.CharField(
        max_length=255, help_text="数据归属部门", db_index=True, null=True, blank=True, verbose_name="数据归属部门"
    )
    update_datetime = models.DateTimeField(
        auto_now=True, null=True, blank=True, db_index=True, help_text="修改时间", verbose_name="修改时间"
    )
    create_datetime = models.DateTimeField(
        auto_now_add=True, null=True, blank=True, db_index=True, help_text="创建时间", verbose_name="创建时间"
    )
    description = models.TextField(verbose_name="描述", null=True, blank=True, help_text="描述")

    class Meta:
        abstract = True
        verbose_name = "核心模型"
        app_label = app_label_db_name
        verbose_name_plural = verbose_name


class BaseTemplate(CoreModel):
    name = models.CharField(
        verbose_name="模版名称", max_length=255, default="", null=False, blank=False, help_text="模版名称"
    )
    top_image = models.CharField(max_length=225, blank=True, null=True, verbose_name="顶部图片", help_text="顶部图片")
    sort = models.IntegerField(default=1, verbose_name="排序", help_text="排序")
    desc = models.TextField(verbose_name="描述", default="", null=True, blank=True, help_text="描述")

    class Meta:
        abstract = True
        verbose_name = verbose_name_plural = "核心模型"


class AddPostgresPartitionedBase:
    """
    pgsql表分表基类
    """

    @classmethod
    def add_hash_partition(cls, number=36):
        """
        创建分区表
        :return:
        """
        if cls.PartitioningMeta.method != "hash":
            raise ProgrammingError("表分区错误，无法进行分区")
        schema_editor = connection.schema_editor()
        for item in range(number):
            try:
                schema_editor.add_hash_partition(
                    model=cls,
                    name="_" + str(item),
                    modulus=number,
                    remainder=item,
                )
            except ProgrammingError as e:
                print(f"{cls.__name__}分表失败：" + str(e).rstrip("\n"))
        return

    @classmethod
    def add_range_day_partition(cls, day=7):
        """
        按照创建时间"天"分表
        :return:
        """
        if cls.PartitioningMeta.method != "range":
            raise ProgrammingError("表分区错误，无法进行分区")
        day_before = date.today().strftime("%Y-%m-%d")
        schema_editor = connection.schema_editor()

        for index in range(day):
            try:
                day_following = (date.today() + timedelta(days=index + 1)).strftime("%Y-%m-%d")
                schema_editor.add_range_partition(
                    model=cls,
                    name=f"{day_before}_{day_following}",
                    from_values=day_before,
                    to_values=day_following,
                )
                day_before = day_following
            except ProgrammingError as e:
                print(f"{cls.__name__}分表失败：" + str(e).rstrip("\n"))
        return

    @classmethod
    def add_range_month_partition(cls, start_date, end_date):
        """
        按照创建时间"月"分表
        :return:
        """
        if cls.PartitioningMeta.method != "range":
            raise ProgrammingError("表分区错误，无法进行分区")
        range_month_partition_list = get_month_range(start_date, end_date)
        schema_editor = connection.schema_editor()
        for index, ele in enumerate(range_month_partition_list):
            if index == 0:
                continue
            try:
                schema_editor.add_range_partition(
                    model=cls,
                    name=f"{range_month_partition_list[index - 1][:-3]}_{ele[:-3]}",
                    from_values=range_month_partition_list[index - 1],
                    to_values=ele,
                )
            except ProgrammingError as e:
                print(f"{cls.__name__}分表失败：" + str(e).rstrip("\n"))
        return

    @classmethod
    def add_list_partition(cls, unique_value):
        """
        按照某个值进行分区
        :param unique_value:
        :return:
        """
        if cls.PartitioningMeta.method != "list":
            raise ProgrammingError("表分区错误，无法进行分区")
        schema_editor = connection.schema_editor()
        try:
            schema_editor.add_list_partition(
                model=cls,
                name=f"_{unique_value}",
                values=[unique_value],
            )
        except ProgrammingError as e:
            print(f"{cls.__name__}分表失败：" + str(e).rstrip("\n"))
        return


def get_all_models_objects(model_name=None):
    """
    获取所有 models 对象
    :return: {}
    """
    settings.ALL_MODELS_OBJECTS = {}
    if not settings.ALL_MODELS_OBJECTS:
        all_models = apps.get_models()
        for item in list(all_models):
            table = {"tableName": item._meta.verbose_name, "table": item.__name__, "tableFields": []}
            for field in item._meta.fields:
                fields = {"title": field.verbose_name, "field": field.name}
                table["tableFields"].append(fields)
            settings.ALL_MODELS_OBJECTS.setdefault(item.__name__, {"table": table, "object": item})
    if model_name:
        return settings.ALL_MODELS_OBJECTS[model_name] or {}
    return settings.ALL_MODELS_OBJECTS or {}



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：django_base_ai
@File    ：deepseek.py
@Desc    ：DeepSeek 大模型调用工具类（OpenAI 兼容接口）
"""

from django_base_ai.utils.ai.openai_compat import OpenAICompatClient

# DeepSeek 官方 API 端点
DEEPSEEK_BASE_URL = "https://api.deepseek.com/v1"

# 常用模型（纯文本 + 视觉）
# deepseek-vl-7b-chat 为视觉语言模型，支持图像输入，需自建或第三方推理；官方 API 若开放 VL 也可用同名
DEEPSEEK_MODELS = (
    "deepseek-chat",
    "deepseek-reasoner",
    "deepseek-coder",
    "deepseek-vl-7b-chat",
)


class DeepSeekClient(OpenAICompatClient):
    """
    DeepSeek 官方大模型客户端。
    使用 OpenAI 兼容接口，需在 DeepSeek 开放平台申请 API Key。
    视觉模型 deepseek-vl-7b-chat：可传 default_model 使用；若官方未开放，需自建/第三方
    兼容接口并设置 base_url，请求体传 model 即可。
    """

    def __init__(
        self,
        api_key: str,
        default_model: str = "deepseek-chat",
        timeout: int = 60,
    ):
        super().__init__(
            api_key=api_key,
            base_url=DEEPSEEK_BASE_URL,
            default_model=default_model,
            timeout=timeout,
        )



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : system_usage.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 系统使用统计量接口
import logging

from rest_framework.decorators import action
from rest_framework.permissions import AllowAny

from django_base_ai.system.models import SystemUsage
from django_base_ai.utils.json_response import DetailResponse
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.viewset import CustomModelViewSet

logger = logging.getLogger(__name__)


class SystemUsageSerializer(CustomModelSerializer):
    class Meta:
        model = SystemUsage
        fields = "__all__"
        read_only_fields = ["id"]


class SystemUsageCreateUpdateSerializer(CustomModelSerializer):
    class Meta:
        model = SystemUsage
        fields = "__all__"


class SystemUsageViewSet(CustomModelViewSet):
    """
    统计量
    list:查询
    create:新增
    update:修改
    retrieve:单例
    destroy:删除
    """

    queryset = SystemUsage.objects.all()
    serializer_class = SystemUsageSerializer
    create_serializer_class = SystemUsageCreateUpdateSerializer
    update_serializer_class = SystemUsageCreateUpdateSerializer
    # extra_filter_backends = []

    @action(methods=["post"], detail=False, permission_classes=[AllowAny], authentication_classes=[])
    def allow_usage(self, request, *args, **kwargs):
        return DetailResponse()



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : base_analyze.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 登录与操作数据分析统计
import json
import logging

from django.conf import settings
from django.core.cache import cache
from django.db.models import Count, Q
from django.db.models.functions import TruncMonth
from django_redis import get_redis_connection
from rest_framework.decorators import action
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.viewsets import ViewSet

from django_base_ai.system.models import Dept, LoginLog, OperationLog, Users
from django_base_ai.utils.datetime_range_generator import get_date_range
from django_base_ai.utils.json_response import DetailResponse

logger = logging.getLogger(__name__)


class AnalyzeView(ViewSet):
    permission_classes = [AllowAny]
    cache_timeout = getattr(settings, "DEPT_LOGIN_CACHE_TIMEOUT", 3600 * 1)

    @action(methods=["get", "post"], detail=False, permission_classes=[AllowAny], authentication_classes=[])
    def read_redis_operation_log(self, request, *args, **kwargs):
        """
        同步请求日志数据接口
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        # 获取 Redis 连接
        redis_conn = get_redis_connection("default")
        operation_log_list = list()
        for i in range(1000):
            popped_item = redis_conn.rpop("redis_operation_log")
            if not popped_item:
                break
            popped_item = json.loads(popped_item.decode("utf-8"))
            operation_log_list.append(OperationLog(**popped_item))
        if operation_log_list:
            OperationLog.objects.bulk_create(operation_log_list)
        return Response({"msg": "已完成"})

    @action(methods=["get"], detail=False, permission_classes=[AllowAny], authentication_classes=[])
    def branches_login_statistical(self, request, *args, **kwargs):
        """
        根据分行统计登录次数（按月统计）
        返回格式: [
            {
                "name": "分行名称",
                "count": 总登录次数,
                "months": {
                    "2025-01": 1月登录次数,
                    "2025-02": 2月登录次数,
                    ...
                    "2025-12": 12月登录次数
                }
            }
        ]
        """
        # 获取查询参数
        start_month = request.query_params.get("start_month", None)
        end_month = request.query_params.get("end_month", None)
        start_date, end_date, months, start_month, end_month = get_date_range(start_month, end_month)

        # 缓存设置
        cache_key = f"branches_login_statistical_{start_month}_{end_month}"

        # 尝试从缓存获取数据
        cached_data = cache.get(cache_key)
        if cached_data is not None:
            return DetailResponse(cached_data)

        # 缓存未命中，执行统计逻辑
        branches = Users.objects.values("main_department").distinct().order_by("main_department")

        result = []
        for branch in branches:
            branch_name = branch["main_department"]

            # 获取该分行下的所有部门ID（递归）
            def get_dept_ids(dept_name):
                depts = Dept.objects.filter(name=dept_name).values_list("id", flat=True)
                dept_ids = list(depts)
                children = Dept.objects.filter(parent__name=dept_name).values_list("id", flat=True)
                dept_ids.extend(children)
                return dept_ids

            dept_ids = get_dept_ids(branch_name)
            if not dept_ids:
                continue

            # 获取这些部门下的所有用户ID
            user_ids = Users.objects.filter(dept__id__in=dept_ids).values_list("id", flat=True)

            # 按月统计登录次数
            monthly_stats = (
                LoginLog.objects.filter(creator__in=user_ids, create_datetime__range=(start_date, end_date))
                .annotate(
                    month=TruncMonth("create_datetime")  # 改为使用TruncMonth获取完整的年月
                )
                .values("month")
                .annotate(count=Count("id"))
                .order_by("month")
            )

            # 初始化月份数据
            months_data = {month: 0 for month in months}  # 直接使用YYYY-MM格式

            # 填充实际数据
            for stat in monthly_stats:
                month_key = stat["month"].strftime("%Y-%m")  # 格式化为YYYY-MM
                if month_key in months_data:
                    months_data[month_key] = stat["count"]

            # 计算总登录次数
            total_count = sum(months_data.values())

            # 添加到结果集
            result.append({"name": branch_name, "count": total_count, "months": months_data})

        # 按总登录次数降序排序
        sorted_result = sorted(result, key=lambda x: x["count"], reverse=True)
        # 将结果存入缓存
        cache.set(cache_key, sorted_result, timeout=self.cache_timeout)

        return DetailResponse(sorted_result)

    @action(methods=["get"], detail=False, permission_classes=[AllowAny], authentication_classes=[])
    def dept_login_statistical(self, request, *args, **kwargs):
        """
        根据组织架构统计登录次数（树状结构按月统计）
        返回格式: {
            "start_month": "2025-01",
            "end_month": "2025-08",
            "total": {
                "name": "全部部门",
                "count": 总登录次数,
                "months": {"2025-01": 1月登录次数, ..., "2025-12": 12月登录次数}
            },
            "departments": [
                {
                    "id": 部门ID,
                    "name": "部门名称",
                    "short_name": "部门简称",
                    "count": 总登录次数,
                    "months": {"2025-01": 1月登录次数, ..., "2025-12": 12月登录次数},
                    "children": [子部门...]
                },
                ...
            ]
        }
        """
        # 获取查询参数
        start_month = request.query_params.get("start_month", None)
        end_month = request.query_params.get("end_month", None)
        start_date, end_date, months, start_month, end_month = get_date_range(start_month, end_month)

        # 缓存设置
        cache_key = f"dept_tree_login_statistical_{start_month}_{end_month}"
        cache_timeout = getattr(settings, "DEPT_LOGIN_CACHE_TIMEOUT", 3600 * 1)

        # 尝试从缓存获取数据
        cached_data = cache.get(cache_key)
        if cached_data is not None:
            return DetailResponse(cached_data)

        # 构建部门树状统计
        def build_dept_tree(dept):
            # 获取当前部门及其所有子部门的ID
            dept_ids = [dept.id]
            children = Dept.objects.filter(parent=dept).order_by("sort")

            # 递归处理子部门
            child_nodes = []
            for child in children:
                child_node = build_dept_tree(child)
                child_nodes.append(child_node)
                dept_ids.extend(child_node["all_children"])

            # 获取这些部门下的所有用户ID
            user_ids = Users.objects.filter(dept__id__in=dept_ids).values_list("id", flat=True)

            # 按月统计登录次数（使用TruncMonth获取完整的年月）
            monthly_stats = (
                LoginLog.objects.filter(creator__in=user_ids, create_datetime__range=(start_date, end_date))
                .annotate(
                    month=TruncMonth("create_datetime")  # 改为使用TruncMonth
                )
                .values("month")
                .annotate(count=Count("id"))
                .order_by("month")
            )

            # 初始化月份数据（使用YYYY-MM格式）
            months_data = {month: 0 for month in months}

            # 填充实际数据
            for stat in monthly_stats:
                month_key = stat["month"].strftime("%Y-%m")  # 格式化为YYYY-MM
                if month_key in months_data:
                    months_data[month_key] = stat["count"]

            # 计算总登录次数
            total_count = sum(months_data.values())

            # 构建返回结构
            node = {
                "id": dept.id,
                "name": dept.name,
                "short_name": dept.short_name or "",
                "count": total_count,
                "months": months_data,  # 使用YYYY-MM格式
                "children": child_nodes,
                "all_children": dept_ids,  # 包含所有子部门ID
            }

            return node

        # 从顶级部门开始构建树
        top_depts = Dept.objects.filter(parent__isnull=True).order_by("sort")
        departments = [build_dept_tree(dept) for dept in top_depts]

        # 计算系统总登录数（所有顶级部门之和）
        total_stats = {month: 0 for month in months}  # 使用YYYY-MM格式
        total_count = 0

        for dept in departments:
            total_count += dept["count"]
            for month in total_stats:
                total_stats[month] += dept["months"].get(month, 0)

        # 构建最终返回结构
        response_data = {
            "start_month": start_month,
            "end_month": end_month,
            "total": {
                "name": "全部部门",
                "count": total_count,
                "months": total_stats,  # 使用YYYY-MM格式
            },
            "departments": departments,
        }

        # 将结果存入缓存
        cache.set(cache_key, response_data, timeout=cache_timeout)

        return DetailResponse(response_data)

    @action(methods=["get"], detail=False, permission_classes=[AllowAny], authentication_classes=[])
    def operation_log_statistical(self, request, *args, **kwargs):
        """
        操作日志统计分析（支持自定义起止月份）
        返回格式: {
            "start_month": "2025-01",
            "end_month": "2025-08",
            "data": [
                {
                    "request_path": "/docs",
                    "display_name": "向日葵",
                    "months": {"2025-01": 访问次数, ...},
                    "total": 总访问次数（从高到低排序）
                },
                ...
            ]
        }
        """
        # 路径映射配置
        path_mapping = getattr(settings, "PATH_MAPPING", {})
        # 获取查询参数
        start_month = request.query_params.get("start_month", None)
        end_month = request.query_params.get("end_month", None)
        start_date, end_date, months, start_month, end_month = get_date_range(start_month, end_month)
        # 缓存设置
        cache_key = f"oplog_stats_{start_month}_{end_month}_v2"
        cache_timeout = getattr(settings, "OPLOG_STATS_CACHE_TIMEOUT", 3600 * 6)  # 默认缓存6小时
        # 尝试从缓存获取数据
        cached_data = cache.get(cache_key)
        if cached_data is not None:
            return DetailResponse(cached_data)
        # 初始化结果数据
        result_data = []
        months_template = {month: 0 for month in months}
        # 批量查询优化
        for path, display_name in path_mapping.items():
            # 按月统计该路径的访问量
            monthly_stats = (
                OperationLog.objects.filter(~Q(request_path=None))
                .filter(Q(request_path__startswith=path) & Q(create_datetime__range=(start_date, end_date)))
                .annotate(month=TruncMonth("create_datetime"))
                .values("month")
                .annotate(count=Count("id"))
                .order_by("month")
            )
            # 填充月份数据
            months_data = months_template.copy()
            total = 0
            for stat in monthly_stats:
                month_key = stat["month"].strftime("%Y-%m")
                if month_key in months_data:  # 只包含在查询范围内的月份
                    months_data[month_key] = stat["count"]
                    total += stat["count"]
            # 添加到结果集
            result_data.append(
                {"request_path": path, "display_name": display_name, "months": months_data, "total": total}
            )
        # 按total从高到低排序
        sorted_data = sorted(result_data, key=lambda x: x["total"], reverse=True)
        # 构建最终响应数据
        response_data = {"start_month": start_month, "end_month": end_month, "data": sorted_data}
        # 设置缓存
        cache.set(cache_key, response_data, timeout=cache_timeout)
        return DetailResponse(response_data)

    @action(methods=["get"], detail=False, permission_classes=[AllowAny], authentication_classes=[])
    def new_users_statistical(self, request, *args, **kwargs):
        """
        新增用户统计分析（基于首次登录时间）
        统计每个月新增的登录用户数（去重统计）
        返回格式: {
            "start_month": "2025-01",
            "end_month": "2025-08",
            "total_users": 总用户数,
            "data": [
                {
                    "month": "2025-01",
                    "new_users_count": 当月新增用户数,
                    "cumulative_count": 累计用户数
                },
                ...
            ]
        }
        """
        # 获取查询参数
        start_month = request.query_params.get("start_month", None)
        end_month = request.query_params.get("end_month", None)
        start_date, end_date, months, start_month, end_month = get_date_range(start_month, end_month)

        # 缓存设置
        cache_key = f"new_users_statistical_{start_month}_{end_month}"

        # 尝试从缓存获取数据
        cached_data = cache.get(cache_key)
        if cached_data is not None:
            return DetailResponse(cached_data)

        # 获取每个用户的首次登录时间
        # 使用子查询找出每个用户的最早登录时间
        from django.db.models import Min

        # 获取每个用户的首次登录记录
        (
            LoginLog.objects.values("creator")
            .annotate(first_login=Min("create_datetime"))
            .values("creator", "first_login")
        )

        # 统计在查询时间范围内首次登录的用户，按月分组
        monthly_new_users = (
            LoginLog.objects.filter(create_datetime__range=(start_date, end_date))
            .values("creator")
            .annotate(first_login=Min("create_datetime"))
            .annotate(month=TruncMonth("first_login"))
            .values("month")
            .annotate(new_users_count=Count("creator", distinct=True))
            .order_by("month")
        )

        # 初始化月份数据
        months_data = []
        cumulative_count = 0

        # 获取查询时间范围之前的用户总数（作为基数）
        before_start_users = (
            LoginLog.objects.filter(create_datetime__lt=start_date).values("creator").distinct().count()
        )

        cumulative_count = before_start_users

        # 创建月份统计数据字典，方便查找
        monthly_stats_dict = {}
        for stat in monthly_new_users:
            month_key = stat["month"].strftime("%Y-%m")
            monthly_stats_dict[month_key] = stat["new_users_count"]

        # 生成完整的月份数据
        for month in months:
            new_users_count = monthly_stats_dict.get(month, 0)
            cumulative_count += new_users_count
            months_data.append(
                {"month": month, "new_users_count": new_users_count, "cumulative_count": cumulative_count}
            )
        # 计算总的登录用户数（去重）
        total_users = LoginLog.objects.values("creator").distinct().count()
        # 构建返回数据
        response_data = {
            "start_month": start_month,
            "end_month": end_month,
            "total_users": total_users,
            "before_start_users": before_start_users,
            "data": months_data,
        }
        # 将结果存入缓存
        cache.set(cache_key, response_data, timeout=self.cache_timeout)
        return DetailResponse(response_data)



#!/usr/bin/env python
"""
@Project ：django_base_ai
@File    ：smtp_email.py
@Author  ：congxing.wang
@Date    ：2026/02/25
@Desc    ：SMTP 邮件发送
"""

import logging
import os
import smtplib
import sys
from email.header import Header
from email.mime.application import MIMEApplication
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import formataddr
from pathlib import Path

import django
from jinja2 import Template

from django_base_ai import dispatch

# 获取日志记录器
logger = logging.getLogger(__name__)


class SmtpEmail:
    def __init__(self):

        # SMTP开启SSL，如果配置为None则默认为True（QQ邮箱需要SSL）
        self.smtp_ssl = dispatch.get_system_config_values("message.smtp_ssl")
        if self.smtp_ssl is None:
            self.smtp_ssl = True
        # SMTP服务器IP
        self.email_host = dispatch.get_system_config_values("message.smtp_email_host")
        # SMTP服务器端口
        self.host_port = dispatch.get_system_config_values("message.smtp_email_host_port")
        # SMTP邮箱发件人名称
        self.email_name = dispatch.get_system_config_values("message.smtp_email_name")
        # SMTP邮箱发件人
        self.email_address = dispatch.get_system_config_values("message.smtp_email_address")
        # SMTP邮箱密码
        self.password = dispatch.get_system_config_values("message.smtp_email_password")
        self.check = False
        if self.email_host or self.host_port or self.email_address:
            self.check = True
            logger.info(
                f"SMTP开启SSL={self.smtp_ssl},SMTP服务器IP={self.email_host}, 端口={self.host_port}, 发件人账号={self.email_address},请检查配置-系统配置-系统通知设置项"
            )

    def send(
        self,
        to_mail,
        subject,
        content,
        cc_list=None,
        bcc_list=None,
        file_list=None,
        content_kwargs={},
        new_formataddr=(),
    ):
        """
        发送邮件，支持抄送、密送和附件
        Args:
            to_mail: 收件人邮箱地址，可以是字符串或列表
            subject: 邮件主题
            content: 邮件内容（HTML格式）
            cc_list: 抄送人列表，默认为None
            bcc_list: 密送人列表，默认为None
            file_list: 附件文件路径列表，默认为None
            content_kwargs: 模板参数，用于Jinja2模板渲染
            new_formataddr: 重置发件人信息
        Returns:
            str: 发送结果信息
        """
        if not self.check:
            return f"SMTP服务器IP={self.email_host}, 端口={self.host_port}, 发件人账号={self.email_address},请检查配置-系统配置-系统通知设置项"
        # 初始化参数
        if cc_list is None:
            cc_list = []
        if bcc_list is None:
            bcc_list = []
        if file_list is None:
            file_list = []
        # 处理收件人格式
        if isinstance(to_mail, str):
            to_mail = [to_mail]
        msg_from = self.email_address
        password = self.password
        # 处理模板渲染
        if len(content_kwargs) > 0:
            # 处理邮件标题
            subject_template = Template(subject)  # 生成模板文件
            subject = subject_template.render(content_kwargs)  # 把数据填充到模板里面
            # 处理邮件内容
            content_template = Template(content)  # 生成模板文件
            content = content_template.render(content_kwargs)  # 把数据填充到模板里面
        # 创建邮件对象
        msg = MIMEMultipart()
        # 设置邮件头信息
        logger.debug(f"msg_from={msg_from},self.email_name={self.email_name}")
        msg["From"] = new_formataddr if new_formataddr else formataddr((self.email_name, msg_from))
        msg["To"] = ",".join(to_mail)
        msg["Subject"] = Header(subject, "utf-8").encode()
        # 添加抄送和密送
        if cc_list:
            msg["Cc"] = ",".join(cc_list)
        if bcc_list:
            msg["Bcc"] = ",".join(bcc_list)
        # 添加邮件正文
        msg.attach(MIMEText(content, "html", "utf-8"))
        try:
            # 连接SMTP服务器
            if self.smtp_ssl:
                # 使用SSL连接，添加超时和调试信息
                client = smtplib.SMTP_SSL(self.email_host, self.host_port, timeout=30)
                # 启用调试模式
                # client.set_debuglevel(1)
            else:
                client = smtplib.SMTP(self.email_host, self.host_port, timeout=30)
                # client.set_debuglevel(1)
            # 登录
            if self.password:
                # 密码不等于空,登录服务器
                logger.info(f"正在尝试登录: {msg_from}")
                client.login(msg_from, password)
                logger.info("登录成功")
            # 添加附件
            for file_info in file_list:
                if isinstance(file_info, dict):
                    # 新格式：字典包含 name 和 file_url
                    file_name = file_info.get("name", "")
                    file_url = file_info.get("file_url", "")
                    # 拼接项目当前目录
                    project_root = Path(__file__).resolve().parent.parent.parent.parent
                    # 移除开头的斜杠，然后拼接项目根目录
                    relative_path = file_url.lstrip("/")
                    full_file_path = project_root / relative_path
                    logger.info(f"添加附件: {file_name} -> {full_file_path}")
                    if os.path.exists(full_file_path):
                        with open(full_file_path, "rb") as f:
                            appendix = MIMEApplication(f.read())
                            appendix.add_header(
                                "Content-Disposition", "attachment", filename=Header(file_name, "utf-8").encode()
                            )
                            msg.attach(appendix)
                    else:
                        logger.warning(f"文件不存在 {full_file_path}")
                else:
                    # 旧格式：直接是文件路径字符串
                    file_path = file_info
                    logger.info(f"添加附件: {file_path}")
                    with open(file_path, "rb") as f:
                        appendix = MIMEApplication(f.read())
                        # 获取文件名
                        filename = os.path.basename(file_path)
                        appendix.add_header(
                            "Content-Disposition", "attachment", filename=Header(filename, "utf-8").encode()
                        )
                        msg.attach(appendix)
            # 准备收件人列表（包括抄送和密送）
            all_recipients = to_mail + cc_list + bcc_list
            # 发送邮件
            client.sendmail(msg_from, all_recipients, msg.as_string())
            logger.info("邮件发送成功")

        except smtplib.SMTPAuthenticationError as e:
            logger.error(f"SMTP认证失败: {e}")
            return f"SMTP认证失败: {e}"
        except smtplib.SMTPConnectError as e:
            logger.error(f"SMTP连接失败: {e}")
            return f"SMTP连接失败: {e}"
        except smtplib.SMTPException as e:
            logger.error(f"SMTP异常: {e}")
            return f"SMTP异常: {e}"
        except Exception as e:
            logger.error(f"发生异常: {e}")
            return f"发生异常: {e}"
        finally:
            try:
                client.quit()
                logger.info("SMTP连接已关闭")
            except Exception:
                pass

        return "发送成功"


if __name__ == "__main__":
    BASE_DIR = f"{Path(__file__).resolve().parent.parent}"
    sys.path.append(BASE_DIR)
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "DjangoBaseAi.settings")
    django.setup()
    logger.info(dispatch.get_system_config_values("message.smtp_ssl"))
    smtp_email = SmtpEmail()
    file_list = [{"name": "测试.xlsx", "file_url": "/media/files/76279c8125916b5c4db9b33eb85023eb.xlsx"}]
    logger.info(smtp_email.send("2256807897@qq.com", "邮件标题", "刘会飙", file_list=file_list))



# -*- coding: UTF-8 -*-
# ! /usr/bin/env python
import base64
import os

import Crypto
from Crypto.Cipher import PKCS1_v1_5 as PKCS1_v1_5_cipper
from Crypto.Hash import SHA
from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5

from django_base_ai import settings as config

# 使用 rsa库进行RSA签名和加解密


class RsaUtil:
    PUBLIC_KEY_PATH = os.path.join(config.BASE_DIR, "./django_base_ai/utils/keys/company_rsa_public_key.pem")  # 公钥
    PRIVATE_KEY_PATH = os.path.join(config.BASE_DIR, "./django_base_ai/utils/keys/company_rsa_private_key.pem")  # 私钥

    # 初始化key
    def __init__(self, company_pub_file=PUBLIC_KEY_PATH, company_pri_file=PRIVATE_KEY_PATH):

        if company_pub_file:
            self.company_public_key = RSA.importKey(open(company_pub_file).read())
        if company_pri_file:
            self.company_private_key = RSA.importKey(open(company_pri_file).read())

    def get_max_length(self, rsa_key, encrypt=True):
        """加密内容过长时 需要分段加密 换算每一段的长度.
        :param rsa_key: 钥匙.
        :param encrypt: 是否是加密.
        """
        blocksize = Crypto.Util.number.size(rsa_key.n) / 8
        reserve_size = 11  # 预留位为11
        if not encrypt:  # 解密时不需要考虑预留位
            reserve_size = 0
        maxlength = blocksize - reserve_size
        return maxlength

    # 加密
    def encrypt_by_public_key(self, encrypt_message):
        """使用公钥加密.
        :param encrypt_message: 需要加密的内容.
        加密之后需要对接过进行base64转码
        """
        encrypt_result = b""
        max_length = int(self.get_max_length(self.company_public_key))
        cipher = PKCS1_v1_5_cipper.new(self.company_public_key)
        while encrypt_message:
            input_data = encrypt_message[:max_length]
            encrypt_message = encrypt_message[max_length:]
            out_data = cipher.encrypt(input_data.encode(encoding="utf-8"))
            encrypt_result += out_data
        encrypt_result = base64.b64encode(encrypt_result)
        return encrypt_result

    # 加密 支付方私钥
    def encrypt_by_private_key(self, encrypt_message):
        """使用私钥加密.
        :param encrypt_message: 需要加密的内容.
        加密之后需要对接过进行base64转码
        """
        encrypt_result = ""
        max_length = int(self.get_max_length(self.company_private_key))
        cipher = PKCS1_v1_5_cipper.new(self.company_public_key)
        while encrypt_message:
            input_data = encrypt_message[:max_length]
            encrypt_message = encrypt_message[max_length:]
            out_data = cipher.encrypt(input_data.encode(encoding="utf-8").strip())
            encrypt_result += out_data
        encrypt_result = base64.b64encode(encrypt_result)
        return encrypt_result

    def decrypt_by_public_key(self, decrypt_message):
        """使用公钥解密.
        :param decrypt_message: 需要解密的内容.
        解密之后的内容直接是字符串，不需要在进行转义
        """
        decrypt_result = b""
        max_length = self.get_max_length(self.company_public_key, False)
        decrypt_message = base64.b64decode(decrypt_message)
        cipher = PKCS1_v1_5_cipper.new(self.company_public_key)
        while decrypt_message:
            input_data = decrypt_message[:max_length]
            decrypt_message = decrypt_message[max_length:]
            out_data = cipher.decrypt(input_data.encode(encoding="utf-8"), "")
            decrypt_result += out_data
        return decrypt_result

    def decrypt_by_private_key(self, decrypt_message):
        """使用私钥解密.
        :param decrypt_message: 需要解密的内容.
        解密之后的内容直接是字符串，不需要在进行转义
        """
        decrypt_result = b""
        max_length = int(self.get_max_length(self.company_private_key, False))
        decrypt_message = base64.b64decode(decrypt_message)
        cipher = PKCS1_v1_5_cipper.new(self.company_private_key)
        while decrypt_message:
            input_data = decrypt_message[:max_length]
            decrypt_message = decrypt_message[max_length:]
            out_data = cipher.decrypt(input_data, "")
            decrypt_result += (
                out_data.decode(encoding="utf-8").encode(encoding="utf-8").strip()
            )  # 此处不能使用 str(out_data) 直接转换bytes
        return decrypt_result.decode(encoding="utf-8")

    # 签名 商户私钥 base64转码
    def sign_by_private_key(self, message):
        """私钥签名.
        :param message: 需要签名的内容.
        签名之后，需要转义后输出
        """
        cipher = PKCS1_v1_5.new(self.company_private_key)  # 用公钥签名，会报错 raise TypeError("No private key") 如下
        # if not self.has_private():
        #   raise TypeError("No private key")
        hs = SHA.new(message)
        signature = cipher.sign(hs)
        return base64.b64encode(signature)

    def verify_by_public_key(self, message, signature):
        """公钥验签.
        :param message: 验签的内容.
        :param signature: 对验签内容签名的值（签名之后，会进行b64encode转码，所以验签前也需转码）.
        """
        signature = base64.b64decode(signature)
        cipher = PKCS1_v1_5.new(self.company_public_key)
        hs = SHA.new(message)

        # digest = hashlib.sha1(message).digest()  # 内容摘要的生成方法有很多种，只要签名和解签用的是一样的就可以

        return cipher.verify(hs, signature)


if __name__ == "__main__":
    # message = 'hellworldhellworldhellworldhellhellworldhellworldhellwo'
    """
    message = json.dumps({"user_name": "wangcongxing", "ip": "127.0.0.1", "mac": "mac地址", "machine_name": "机器名称","os_name": "操作系统名称", "cpu_info": "cpuId", "memory_capacity": "内存容量","disk_size": "磁盘大小", "desktop_monitor_name": "桌面显示器名称"})
    print("明文内容：>>> ")
    print(message)
    rsaUtil = RsaUtil()
    encrypy_result = rsaUtil.encrypt_by_public_key(message)
    print("加密结果：>>> ")
    print(encrypy_result)

    #encrypy_result = "H/AjTxneOZijxtfFabUD1bPxIhqCRqY2MsPRo8iBQDwQQZ8N4oVAmAtIoMfIsj7uItpkuCU7RCAGzXl1xHTd6FI2XXH1R1XBHqZ20JjGdhNy4L9PVZlEzw7gWUII9dM15nzrPanuvsM2MV/xQRg5JEcmlf0wSs/1gFGfRZJS+4M="
    decrypt_result = rsaUtil.decrypt_by_private_key(encrypy_result)
    print("解密结果：>>> ")
    print(decrypt_result)
    sign = rsaUtil.sign_by_private_key(bytearray(message.encode(encoding='utf-8')))
    print("签名结果：>>> ")
    print(sign)
    print("验签结果：>>> ")
    print(rsaUtil.verify_by_public_key(bytearray(message.encode(encoding='utf-8')), sign))
    """

    rsa_util = RsaUtil()
    encrypy_result = b"sZFqKFfodTc34X8PadMkoA7uo4h38WQvLVd9wifLE4M5xZQeGrpSRfl4MLwAuR3OSOrbbUT3FyoZY7lLquOfoh3yF0JauxcLJGOGmHH68/Yr8PxsL4q2kH+wd0k5agUEbjL8o+zCKmHrVZqa+/dd6feYmeR8UqcgLx1BaWai7YY="
    decrypt_result = rsa_util.decrypt_by_private_key(encrypy_result)
    print("解密结果：>>> ")
    print(decrypt_result)



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : tags.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 用户自定义标签管理接口
import json
import logging

import httpx
from django.db.models import Q
from rest_framework import serializers
from rest_framework.decorators import action
from rest_framework.permissions import IsAuthenticated

from django_base_ai.system.models import Apps, Tags
from django_base_ai.utils.json_response import DetailResponse, SuccessResponse
from django_base_ai.utils.permission import OpenApiPermission
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.viewset import CustomModelViewSet

logger = logging.getLogger(__name__)


class TagsSerializer(CustomModelSerializer):
    user_info = serializers.SerializerMethodField(read_only=True)
    read_user_info = serializers.SerializerMethodField(read_only=True)
    edit_user_info = serializers.SerializerMethodField(read_only=True)
    dept = serializers.SerializerMethodField(read_only=True)

    def get_user_info(self, obj):
        return list(
            obj.user_info.all().values(
                "id", "username", "avatar", "name", "employee_no", "dept__short_name", "dept__name"
            )
        )

    def get_read_user_info(self, obj):
        return list(
            obj.read_user_info.all().values(
                "id", "username", "avatar", "name", "employee_no", "dept__short_name", "dept__name"
            )
        )

    def get_edit_user_info(self, obj):
        return list(
            obj.edit_user_info.all().values(
                "id", "username", "avatar", "name", "employee_no", "dept__short_name", "dept__name"
            )
        )

    def get_dept(self, obj):
        return list(obj.dept.all().values("id", "name", "short_name", "owner_user", "status"))

    class Meta:
        model = Tags
        fields = "__all__"
        read_only_fields = ["id"]


class TagsCreateSerializer(CustomModelSerializer):
    class Meta:
        model = Tags
        fields = "__all__"


class TagsUpdateSerializer(CustomModelSerializer):
    class Meta:
        model = Tags
        fields = "__all__"


class TagsInitSerializer(CustomModelSerializer):
    """
    初始化获取数信息(用于生成初始化json文件)
    """

    def validate_user_info(self, value):
        """验证并过滤不存在的用户ID"""
        if value:
            from django_base_ai.system.models import Users

            # 过滤掉不存在的用户ID
            existing_user_ids = Users.objects.filter(id__in=value).values_list("id", flat=True)
            filtered_ids = list(existing_user_ids)
            if len(filtered_ids) != len(value):
                missing_ids = set(value) - set(filtered_ids)
                logger.warning(f"Tags 初始化时发现不存在的用户ID: {missing_ids}，已自动过滤")
            return filtered_ids
        return value

    class Meta:
        model = Tags
        fields = ["name", "sort", "status", "user_info", "remark"]
        read_only_fields = ["id"]
        extra_kwargs = {}


class TagsViewSet(CustomModelViewSet):
    """
    标签管理接口
    list:查询
    create:新增
    update:修改
    retrieve:单例
    destroy:删除
    """

    queryset = Tags.objects.all()
    serializer_class = TagsSerializer
    create_serializer_class = TagsCreateSerializer
    update_serializer_class = TagsUpdateSerializer

    # extra_filter_backends = []

    def create_or_update_tags(self, instance, action_type):
        apps_result = Apps.objects.filter(~Q(create_or_update_callback_url="")).filter(~Q(delete_callback_url="")).all()
        for item in apps_result:
            try:
                payload = json.dumps(
                    {"node_type": 3, "action_type": action_type, "info": dict(TagsSerializer(instance).data)},
                    ensure_ascii=False,
                )
                httpx.post(
                    item.create_or_update_callback_url,
                    headers={"Content-Type": "application/json; charset=utf-8"},
                    data=payload,
                )
            except Exception as e:
                print(e.args)

    def delete_tags(self, instance, action_type):
        apps_result = Apps.objects.filter(~Q(create_or_update_callback_url="")).filter(~Q(delete_callback_url="")).all()
        payload = json.dumps(
            {"node_type": 3, "action_type": action_type, "info": dict(TagsSerializer(instance).data)},
            ensure_ascii=False,
        )
        for item in apps_result:
            try:
                httpx.post(
                    item.delete_callback_url, headers={"Content-Type": "application/json; charset=utf-8"}, data=payload
                )
            except Exception as e:
                print(e.args)

    def perform_create(self, serializer):
        """
        重写 create 方法并在创建时执行回调函数
        """
        instance = serializer.save()
        self.post_create_callback(instance)

    def perform_update(self, serializer):
        """
        重写 update 方法并在更新时执行回调函数
        """
        instance = serializer.save()
        self.post_update_callback(instance)

    def destroy(self, request, *args, **kwargs):
        pk = kwargs.get("pk")
        instance = Tags.objects.filter(id=pk).first()
        self.post_delete_callback(instance)
        instance.delete()
        return DetailResponse(data=[], msg="删除成功")

    def post_create_callback(self, instance):
        """
        创建标签后的回调函数，可以执行日志记录、数据同步等操作
        """
        # 示例：可以在这里添加创建标签后要执行的操作
        print(f"标签 {instance.name} 已创建，执行回调处理。")
        self.create_or_update_tags(instance, 0)

    def post_update_callback(self, instance):
        """
        更新标签后的回调函数，可以执行日志记录、数据同步等操作
        """
        # 示例：可以在这里添加更新标签后要执行的操作
        print(f"标签 {instance.name} 已更新，执行回调处理。")
        self.create_or_update_tags(instance, 1)

    def post_delete_callback(self, instance):
        """
        删除标签后的回调函数，可以执行日志记录、数据同步等操作
        """
        # 示例：可以在这里添加删除标签后要执行的操作
        print(f"标签 {instance.name} 已删除，执行回调处理。")
        self.delete_tags(instance, 2)

    @action(methods=["GET"], detail=False, permission_classes=[OpenApiPermission], authentication_classes=[])
    def open_tags_info(self, request, *args, **kwargs):
        """
        第三方API同步部门接口
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        queryset = self.get_queryset()
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True, request=request)
            # 掩码
            serializer_data = self.set_mask_fields(serializer)
            return self.get_paginated_response(serializer_data)
        serializer = self.get_serializer(queryset, many=True, request=request)
        return SuccessResponse(data=serializer.data, msg="获取成功")

    @action(methods=["get"], detail=False, permission_classes=[IsAuthenticated])
    def other_tags(self, request, *args, **kwargs):
        """
        其它标签
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        result = []
        current_user = request.user
        queryset = (
            Tags.objects.filter(status=True)
            .filter(Q(read_user_info__in=[current_user]) | Q(edit_user_info__in=[current_user]))
            .distinct()
            .order_by("sort")
        )
        for item in queryset:
            status = False
            if item.edit_user_info.all().filter(id=current_user.id).exists():
                status = True
            user_info = item.user_info.all().values("id", "username", "avatar", "name", "employee_no")
            result.append({"id": item.id, "name": item.name, "user_info": user_info, "read_or_edit": status})
        return DetailResponse(result)



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : task_run_history.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 任务运行历史查询接口
import logging

from rest_framework.decorators import action

from django_base_ai.system.models import TaskRunHistory
from django_base_ai.utils.json_response import DetailResponse, ErrorResponse
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.viewset import CustomModelViewSet

logger = logging.getLogger(__name__)


class TaskRunHistorySerializer(CustomModelSerializer):
    class Meta:
        model = TaskRunHistory
        fields = ["id", "url_name", "start_time", "end_time", "status_code", "request_config"]
        read_only_fields = ["id"]


class TaskRunHistoryViewSet(CustomModelViewSet):
    """
    消息中心用户列表接口
    list:查询
    create:新增
    update:修改
    retrieve:单例
    destroy:删除
    """

    queryset = TaskRunHistory.objects.order_by("-create_datetime")
    serializer_class = TaskRunHistorySerializer
    # extra_filter_backends = []
    filter_fields = ["base_task", "periodic_task", "$url_name"]

    # 查询request的结果
    @action(methods=["GET"], detail=False)
    def get_request_result(self, request):
        nid = self.request.query_params.get("id", 0)
        if not nid:
            return ErrorResponse(msg="参数不合法")
        run_result = TaskRunHistory.objects.get(id=nid).run_result
        return DetailResponse(data={"run_result": run_result}, msg="获取成功")

    # 根据时间查询
    def get_queryset(self):
        queryset = super().get_queryset()
        # 获取查询参数
        base_task = self.request.GET.get("base_task", 0)
        periodic_task = self.request.GET.get("periodic_task", 0)
        start_date = self.request.GET.get("start_date", "")
        end_date = self.request.GET.get("end_date", "")
        if periodic_task:
            # 定时器的查询
            queryset = queryset.filter(base_task=base_task, periodic_task=periodic_task)
        else:
            # 手动执行的查询
            queryset = queryset.filter(base_task=base_task, periodic_task__isnull=True)
        # 时间过滤
        if start_date and end_date:
            queryset = queryset.filter(end_time__range=(start_date, end_date))
        elif start_date:
            queryset = queryset.filter(end_time__gte=start_date)
        elif end_date:
            queryset = queryset.filter(end_time__lte=end_date)
        return queryset



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : my_superiors.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 我的上级领导管理接口
import logging

from django_base_ai.system.models import MySuperiors
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.viewset import CustomModelViewSet

logger = logging.getLogger(__name__)


class MySuperiorsSerializer(CustomModelSerializer):
    class Meta:
        model = MySuperiors
        fields = "__all__"
        read_only_fields = ["id"]


class MySuperiorsCreateUpdateSerializer(CustomModelSerializer):
    class Meta:
        model = MySuperiors
        fields = "__all__"


class MySuperiorsViewSet(CustomModelViewSet):
    """
    协作人
    list:查询
    create:新增
    update:修改
    retrieve:单例
    destroy:删除
    """

    queryset = MySuperiors.objects.all()
    serializer_class = MySuperiorsSerializer



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : custome_login.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 自定义登录接口
import logging

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.cache import cache
from rest_framework.views import APIView
from rest_framework_simplejwt.tokens import RefreshToken

from django_base_ai import dispatch
from django_base_ai.system.models import Role
from django_base_ai.utils.json_response import DetailResponse
from django_base_ai.utils.request_util import save_login_log

User = get_user_model()

# 创建日志记录器
logger = logging.getLogger(__name__)


class CustomeLoginJWTAPIView(APIView):
    authentication_classes = ()
    permission_classes = ()

    # 用户自定义登录
    def post(self, request, *args, **kwargs):
        """
        执行自定义登录代码
        :param request: 用户请求体
        :param args:
        :param kwargs:
        :return:
        """
        logger.info("开始执行自定义登录")
        logger.debug(f"系统配置: {settings.SYSTEM_CONFIG}")

        # 默认
        func_str = ""
        namespace = {}

        try:
            fun = compile(func_str, "<string>", "exec")
            exec(fun, namespace)
            logger.info("自定义登录函数编译成功")

            ret = namespace["run_cmd"](kwargs=request.data)
            logger.info(f"自定义登录函数执行结果: {ret}")

            default_role_ids = dispatch.get_system_config_values("third.default_role")
            logger.debug(f"获取到的默认角色IDs: {default_role_ids}")

            user_id = ret.get("id")
            logger.debug(f"从返回结果中获取的用户ID: {user_id}")

            user = User.objects.get(id=user_id)
            logger.info(f"查询到用户: {user.username}, ID: {user.id}")

            # 新增到普通用户组
            roles = Role.objects.filter(id__in=default_role_ids)
            user.role.add(*roles)
            logger.info(f"已将用户添加到角色: {[r.name for r in roles]}")

            # 获得用户后，校验密码并签发token
            refresh = RefreshToken.for_user(user)
            cache.set(f"user_{user.id}", "online", 60 * 5)
            access_token = str(refresh.access_token)

            logger.info(f"生成Token成功, 用户ID: {user.id}")

            data = {
                "access": access_token,
                "refresh": str(refresh),
                "name": user.name,
                "skin": user.skin,
                "userId": user.id,
                "is_staff": user.is_staff,
                "avatar": user.avatar,
                "pwd_defaulted": user.pwd_defaulted,
                "user_type": user.user_type,
                "role_info": user.role.values("id", "name", "key"),
            }

            user.last_token = access_token
            user.save()
            logger.debug("用户Token已保存到数据库")

            # 记录登录日志
            save_login_log(request=request, login_type=900, current_user=user)
            logger.info(f"自定义登录成功, 用户: {user.username}, 登录类型: 900")

            return DetailResponse(data)
        except Exception as e:
            logger.error(f"自定义登录过程中发生错误: {str(e)}", exc_info=True)
            raise



"""
DRF 标准 JSON 响应封装（Rules/python.mdc: 类型注解与 docstrings）。

提供 SuccessResponse（分页）、DetailResponse（单条）、ErrorResponse（错误）。
"""

from __future__ import annotations

from typing import Any

from rest_framework.response import Response


class SuccessResponse(Response):
    """
    标准成功响应，带分页结构。

    用法: SuccessResponse(data=...) 或 SuccessResponse(data, page=1, limit=10, total=100)。
    默认 code=2000，不对外支持其他返回码。
    """

    def __init__(
        self,
        data: Any = None,
        msg: str = "success",
        status: int | None = None,
        template_name: str | None = None,
        headers: dict[str, str] | None = None,
        exception: bool = False,
        content_type: str | None = None,
        page: int = 1,
        limit: int = 1,
        total: int = 1,
    ) -> None:
        std_data: dict[str, Any] = {
            "code": 2000,
            "data": {"page": page, "limit": limit, "total": total, "data": data},
            "msg": msg,
        }
        super().__init__(std_data, status, template_name, headers, exception, content_type)


class DetailResponse(Response):
    """
    不包含分页的成功响应，用于单条数据查询。

    用法: DetailResponse(data=...)。
    默认 code=2000。
    """

    def __init__(
        self,
        data: Any = None,
        msg: str = "success",
        status: int | None = None,
        template_name: str | None = None,
        headers: dict[str, str] | None = None,
        exception: bool = False,
        content_type: str | None = None,
    ) -> None:
        std_data: dict[str, Any] = {"code": 2000, "data": data, "msg": msg}
        super().__init__(std_data, status, template_name, headers, exception, content_type)


class ErrorResponse(Response):
    """
    标准错误响应。

    用法: ErrorResponse(msg="xxx") 或 ErrorResponse(code=404, msg="未找到")。
    默认 code=400。
    """

    def __init__(
        self,
        data: Any = None,
        msg: str = "error",
        code: int = 400,
        status: int | None = None,
        template_name: str | None = None,
        headers: dict[str, str] | None = None,
        exception: bool = False,
        content_type: str | None = None,
    ) -> None:
        std_data: dict[str, Any] = {"code": code, "data": data, "msg": msg}
        super().__init__(std_data, status, template_name, headers, exception, content_type)



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : frequently_used_contacts.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 常用联系人管理接口
import logging

from rest_framework import serializers

from django_base_ai.system.models import FrequentlyUsedContacts
from django_base_ai.utils.json_response import DetailResponse
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.viewset import CustomModelViewSet

logger = logging.getLogger(__name__)


class FrequentlyUsedContactsSerializer(CustomModelSerializer):
    """
    常用联系人
    """

    avatar = serializers.SerializerMethodField(read_only=True)
    name = serializers.SerializerMethodField(read_only=True)

    def get_avatar(self, instance):
        return instance.user.avatar

    def get_name(self, instance):
        return f"{instance.user.name}({instance.user.username.upper()})"

    class Meta:
        model = FrequentlyUsedContacts
        fields = "__all__"
        read_only_fields = ["id"]


class FrequentlyUsedContactsCreateSerializer(CustomModelSerializer):
    class Meta:
        model = FrequentlyUsedContacts
        fields = "__all__"


class FrequentlyUsedContactsUpdateSerializer(CustomModelSerializer):
    class Meta:
        model = FrequentlyUsedContacts
        fields = "__all__"


class FrequentlyUsedContactsViewSet(CustomModelViewSet):
    """
    常用联系人
    list:查询
    create:新增
    update:修改
    retrieve:单例
    destroy:删除
    """

    queryset = FrequentlyUsedContacts.objects.all()
    serializer_class = FrequentlyUsedContactsSerializer
    create_serializer_class = FrequentlyUsedContactsCreateSerializer
    update_serializer_class = FrequentlyUsedContactsUpdateSerializer

    # extra_filter_backends = []

    def create(self, request, *args, **kwargs):
        uid = request.data.get("user")
        is_flag = FrequentlyUsedContacts.objects.filter(user_id=uid, creator=self.request.user.id).exists()
        if is_flag:
            return DetailResponse()
        return super().create(request, *args, **kwargs)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
import logging

from django.conf import settings
from django.core.management.base import BaseCommand

logger = logging.getLogger(__name__)


def _app_module_name(app_label):
    """从 INSTALLED_APPS 项解析出可导入的 app 模块名（用于 fixtures.initialize）。"""
    if ".apps." in app_label:
        return app_label.split(".apps.")[0]
    return app_label


def _has_fixtures_initialize(module_name):
    """仅当存在 {module_name}.fixtures.initialize 时才返回 True，避免对无该结构的 app 报错。"""
    try:
        from importlib.util import find_spec

        return find_spec(f"{module_name}.fixtures.initialize") is not None
    except (ModuleNotFoundError, ValueError):
        return False


class Command(BaseCommand):
    """
    项目初始化命令: python manage.py init
    使用当前项目的 settings（django.conf.settings），确保 INSTALLED_APPS 等配置来自主工程。
    仅对提供 fixtures.initialize 的 app 执行初始化，其余 app 静默跳过。
    """

    def add_arguments(self, parser):
        parser.add_argument(
            "init_name",
            nargs="*",
            type=str,
        )
        parser.add_argument("-y", nargs="*")
        parser.add_argument("-Y", nargs="*")
        parser.add_argument("-n", nargs="*")
        parser.add_argument("-N", nargs="*")

    def handle(self, *args, **options):
        reset = False
        if isinstance(options.get("y"), list) or isinstance(options.get("Y"), list):
            reset = True
        if isinstance(options.get("n"), list) or isinstance(options.get("N"), list):
            reset = False
        for app in settings.INSTALLED_APPS:
            module_name = _app_module_name(app)
            if not _has_fixtures_initialize(module_name):
                continue
            try:
                exec(
                    f"""
from {module_name}.fixtures.initialize import Initialize
Initialize(reset={reset},app="{app}").run()
                """
                )
            except Exception as e:
                logger.exception("初始化 %s 失败: %s", app, e)
                self.stderr.write(self.style.ERROR(f"初始化 {app} 失败: {e}"))
        self.stdout.write(self.style.SUCCESS("初始化数据完成！"))



#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : im_chat_question_group.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 热门问题分组
import logging

from rest_framework import serializers

from django_base_ai.system.models import IMChatQuestion, IMChatQuestionGroup
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.viewset import CustomModelViewSet

logger = logging.getLogger(__name__)


class IMChatQuestionGroupSerializer(CustomModelSerializer):
    question_count = serializers.SerializerMethodField()

    def get_question_count(self, obj):
        return IMChatQuestion.objects.filter(im_chat_question_group=obj).count()

    class Meta:
        model = IMChatQuestionGroup
        fields = ["id", "title", "question_count"]
        read_only_fields = ["id"]


class IMChatQuestionCreateUpdateSerializer(CustomModelSerializer):
    class Meta:
        model = IMChatQuestionGroup
        fields = "__all__"
        read_only_fields = ["id"]


class IMChatQuestionGroupViewSet(CustomModelViewSet):
    """
    员工服务的热门问题分组
    list:查询
    create:新增
    update:修改
    retrieve:单例
    destroy:删除
    """

    queryset = IMChatQuestionGroup.objects.all().order_by("sort")
    serializer_class = IMChatQuestionGroupSerializer
    filter_fields = ["ai_desk_app_manage"]
    search_fields = ["title"]
    create_serializer_class = IMChatQuestionCreateUpdateSerializer
    update_serializer_class = IMChatQuestionCreateUpdateSerializer
    # extra_filter_backends = []



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
import hashlib
import logging

from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend
from django.contrib.auth.hashers import check_password
from django.utils import timezone

from django_base_ai.utils.validator import CustomValidationError

logger = logging.getLogger(__name__)
UserModel = get_user_model()


class CustomBackend(ModelBackend):
    """
    Django原生认证方式
    """

    def authenticate(self, request, username=None, password=None, **kwargs):
        msg = f"{username} 正在使用本地登录..."
        logger.info(msg)
        if username is None:
            username = kwargs.get(UserModel.USERNAME_FIELD)
        try:
            user = UserModel._default_manager.get_by_natural_key(username)
        except UserModel.DoesNotExist:
            UserModel().set_password(password)
        else:
            verify_password = check_password(password, user.password)
            if not verify_password:
                password = hashlib.md5(password.encode(encoding="UTF-8")).hexdigest()
                verify_password = check_password(password, user.password)
            if verify_password:
                if self.user_can_authenticate(user):
                    user.last_login = timezone.now()
                    user.save()
                    return user
                raise CustomValidationError("当前用户已被禁用，请联系管理员!")



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：django_base_ai
@File    ：settings_conf.py
@Desc    ：从 conf/ Django settings 读取 AI 相关配置（api_key 等）
"""

import logging

from django.conf import settings

logger = logging.getLogger(__name__)


def _get_setting(key: str, default: str = "") -> str:
    return (getattr(settings, key, None) or default) or ""


def get_ai_config(provider: str) -> dict:
    """
    从 conf 注入的 Django settings 中读取指定供应商的配置。
    约定：在 conf/env.py、conf/pro.py 等中设置 AI_<PROVIDER>_API_KEY 等。
    :param provider: deepseek / tencent / openai / openai_custom
    :return: dict 含 api_key, base_url(可选), default_model(可选)
    """
    prefix = f"AI_{provider.upper()}_"
    api_key = _get_setting(f"{prefix}API_KEY")
    base_url = _get_setting(f"{prefix}BASE_URL")
    default_model = _get_setting(f"{prefix}DEFAULT_MODEL")
    logger.debug("get_ai_config provider=%s base_url=%s", provider, base_url or "(default)")
    return {
        "api_key": api_key,
        "base_url": base_url or None,
        "default_model": default_model or None,
    }



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：django_base_ai
@File    ：base.py
@Desc    ：AI 客户端抽象基类与通用数据结构
"""

from abc import ABC, abstractmethod
from dataclasses import dataclass, field
from typing import Any, Union


@dataclass
class ModelItem:
    """自定义供应商下的单个模型。"""

    model_id: str  # 请求时使用的模型 ID，如 deepseek-chat
    name: str = ""  # 展示名称，如 DeepSeek Chat


@dataclass
class CustomProviderConfig:
    """
    自定义 API 配置，与前端「编辑供应商」表单对应。
    支持 API 格式（如 OpenAI）、前置 URL、模型列表等。
    """

    name: str
    api_key: str
    base_url: str  # API 前置 URL，如 https://api.deepseek.com/v1
    api_format: str = "OpenAI"  # 如 OpenAI
    models: list[ModelItem] = field(default_factory=list)
    default_model: str | None = None  # 默认使用的 model_id，不传则取 models[0].model_id


@dataclass
class ChatMessage:
    """单条对话消息。content 为 str 或多模态 list（如 [{"type":"text","text":"..."},{"type":"image_url","image_url":{"url":"data:..."}}]）。"""

    role: str  # system / user / assistant
    content: Union[str, list[Any]]  # 纯文本或 OpenAI 多模态 content 数组


@dataclass
class CompletionResult:
    """一次 completion 的返回结果。"""

    content: str
    model: str = ""
    usage: dict[str, int] = field(default_factory=dict)
    raw: Any = None
    finish_reason: str = ""


class ChatSession:
    """
    带会话上下文的对话封装，自动累积 user/assistant 消息，多轮交互保持上下文。
    """

    def __init__(
        self,
        client: "BaseAIClient",
        system_prompt: str | None = None,
        max_history_turns: int | None = None,
    ):
        """
        :param client: AI 客户端
        :param system_prompt: 可选系统提示，置于对话开头
        :param max_history_turns: 可选，保留最近 N 轮（每轮 user+assistant），超出则从最早开始丢弃，避免上下文过长
        """
        self._client = client
        self._messages: list[ChatMessage] = []
        if system_prompt:
            self._messages.append(ChatMessage(role="system", content=system_prompt))
        self._max_history_turns = max_history_turns

    @property
    def messages(self) -> list[ChatMessage]:
        """当前会话内的全部消息（含 system/user/assistant）。"""
        return self._messages.copy()

    def send(self, user_content: str, model: str | None = None, **kwargs: Any) -> CompletionResult:
        """
        发送用户内容，调用模型并自动将会话上下文（含本次 user 与 assistant 回复）写入会话。
        :param user_content: 用户本轮输入
        :param model: 可选模型名
        :param kwargs: 透传给 client.chat 的参数（如 temperature、max_tokens）
        :return: 本轮的 CompletionResult
        """
        self._messages.append(ChatMessage(role="user", content=user_content))
        if self._max_history_turns is not None:
            self._trim_to_max_turns()
        result = self._client.chat(self._messages, model=model, **kwargs)
        self._messages.append(ChatMessage(role="assistant", content=result.content))
        if self._max_history_turns is not None:
            self._trim_to_max_turns()
        return result

    def _trim_to_max_turns(self) -> None:
        """保留 system（若有）+ 最近 max_history_turns 轮（每轮 user+assistant）。"""
        if self._max_history_turns is None or self._max_history_turns <= 0:
            return
        system_msgs = [m for m in self._messages if m.role == "system"]
        rest = [m for m in self._messages if m.role != "system"]
        keep = self._max_history_turns * 2
        if len(rest) > keep:
            rest = rest[-keep:]
        self._messages = system_msgs + rest

    def clear(self) -> None:
        """清空会话上下文（若构造时带 system_prompt，会保留一条 system 消息）。"""
        system_msgs = [m for m in self._messages if m.role == "system"]
        self._messages = system_msgs.copy()


class BaseAIClient(ABC):
    """AI 调用客户端抽象基类，统一 chat/completion 接口。"""

    @abstractmethod
    def chat(
        self,
        messages: list[ChatMessage],
        model: str | None = None,
        **kwargs: Any,
    ) -> CompletionResult:
        """
        发送对话请求并返回补全结果。
        :param messages: 消息列表，通常含 system/user/assistant
        :param model: 模型名，由子类默认值或实现决定
        :param kwargs: 厂商扩展参数（temperature、max_tokens 等）
        :return: CompletionResult
        """
        pass

    def test_connection(self) -> tuple[bool, str]:
        """
        测试当前配置是否可用（如校验 API Key / 前置 URL）。
        :return: (是否成功, 消息)
        """
        try:
            self.chat([ChatMessage(role="user", content="hi")], max_tokens=2)
            return True, ""
        except Exception as e:
            return False, str(e)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
import logging
from types import FunctionType, MethodType
from urllib.parse import quote

from django.http import HttpResponse

logger = logging.getLogger(__name__)
from openpyxl import Workbook
from openpyxl.utils import get_column_letter, quote_sheetname
from openpyxl.worksheet.datavalidation import DataValidation
from openpyxl.worksheet.table import Table, TableStyleInfo
from rest_framework.decorators import action
from rest_framework.request import Request

from django_base_ai.utils.import_export import import_to_data
from django_base_ai.utils.json_response import DetailResponse
from django_base_ai.utils.request_util import get_verbose_name


class ImportSerializerMixin:
    """
    自定义导入模板、导入功能
    """

    # 导入字段
    import_field_dict = {}
    # 导入序列化器
    import_serializer_class = None
    # 表格表头最大宽度，默认50个字符
    export_column_width = 50

    def is_number(self, num):
        try:
            float(num)
            return True
        except ValueError:
            pass

        try:
            import unicodedata

            unicodedata.numeric(num)
            return True
        except (TypeError, ValueError):
            pass
        return False

    def get_string_len(self, string):
        """
        获取字符串最大长度
        :param string:
        :return:
        """
        length = 4
        if string is None:
            return length
        if self.is_number(string):
            return length
        for char in string:
            length += 2.1 if ord(char) > 256 else 1
        return round(length, 1) if length <= self.export_column_width else self.export_column_width

    @action(methods=["get", "post"], detail=False)
    # @transaction.atomic  # Django 事务,防止出错 数据量大会报错
    def import_data(self, request: Request, *args, **kwargs):
        """
        导入模板
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        assert self.import_field_dict, f"'{self.__class__.__name__}' 请配置对应的导出模板字段。"
        if isinstance(self.import_field_dict, MethodType) or isinstance(self.import_field_dict, FunctionType):
            self.import_field_dict = self.import_field_dict()
        # 导出模板
        if request.method == "GET":
            # 示例数据
            queryset = self.filter_queryset(self.get_queryset())
            # 导出excel 表
            response = HttpResponse(content_type="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet")
            response["Access-Control-Expose-Headers"] = "Content-Disposition"
            response["Content-Disposition"] = (
                f"attachment;filename={quote(str(f'导入{get_verbose_name(queryset)}模板.xlsx'))}"
            )
            wb = Workbook()
            ws1 = wb.create_sheet("data", 1)
            ws1.sheet_state = "hidden"
            ws = wb.active
            row = get_column_letter(len(self.import_field_dict) + 1)
            column = 10
            header_data = [
                "序号",
            ]
            validation_data_dict = {}
            for index, ele in enumerate(self.import_field_dict.values()):
                if isinstance(ele, dict):
                    header_data.append(ele.get("title"))
                    choices = ele.get("choices", {})
                    if choices.get("data"):
                        data_list = []
                        data_list.extend(choices.get("data").keys())
                        validation_data_dict[ele.get("title")] = data_list
                    elif choices.get("queryset") and choices.get("values_name"):
                        data_list = choices.get("queryset").values_list(choices.get("values_name"), flat=True)
                        validation_data_dict[ele.get("title")] = list(data_list)
                    else:
                        continue
                    column_letter = get_column_letter(len(validation_data_dict))
                    dv = DataValidation(
                        type="list",
                        formula1=f"{quote_sheetname('data')}!${column_letter}$2:${column_letter}${len(validation_data_dict[ele.get('title')]) + 1}",
                        allow_blank=True,
                    )
                    ws.add_data_validation(dv)
                    dv.add(f"{get_column_letter(index + 2)}2:{get_column_letter(index + 2)}1048576")
                else:
                    header_data.append(ele)
            # 添加数据列
            ws1.append(list(validation_data_dict.keys()))
            for index, validation_data in enumerate(validation_data_dict.values()):
                for inx, ele in enumerate(validation_data):
                    ws1[f"{get_column_letter(index + 1)}{inx + 2}"] = ele
            # 插入导出模板正式数据
            df_len_max = [self.get_string_len(ele) for ele in header_data]
            ws.append(header_data)
            # 　更新列宽
            for index, width in enumerate(df_len_max):
                ws.column_dimensions[get_column_letter(index + 1)].width = width
            tab = Table(displayName="Table1", ref=f"A1:{row}{column}")  # 名称管理器
            style = TableStyleInfo(
                name="TableStyleLight11",
                showFirstColumn=True,
                showLastColumn=True,
                showRowStripes=True,
                showColumnStripes=True,
            )
            tab.tableStyleInfo = style
            ws.add_table(tab)
            wb.save(response)
            return response
        else:
            # 从excel中组织对应的数据结构，然后使用序列化器保存
            queryset = self.filter_queryset(self.get_queryset())
            # 获取多对多字段
            m2m_fields = [
                ele.name
                for ele in queryset.model._meta.get_fields()
                if hasattr(ele, "many_to_many") and ele.many_to_many
            ]
            import_field_dict = {"id": "更新主键(勿改)", **self.import_field_dict}
            data = import_to_data(request.data.get("url"), import_field_dict, m2m_fields)
            logger.info("导入开始 model=%s 条数=%d", queryset.model._meta.label, len(data))
            for ele in data:
                filter_dic = {"id": ele.get("id")}
                instance = filter_dic and queryset.filter(**filter_dic).first()
                serializer = self.import_serializer_class(instance, data=ele, request=request)
                serializer.is_valid(raise_exception=True)
                serializer.save()
            logger.info("导入完成 model=%s", queryset.model._meta.label)
            return DetailResponse(msg="导入成功！")

    @action(methods=["get"], detail=False)
    def update_template(self, request):
        queryset = self.filter_queryset(self.get_queryset())
        assert self.import_field_dict, f"'{self.__class__.__name__}' 请配置对应的导入模板字段。"
        assert self.import_serializer_class, f"'{self.__class__.__name__}' 请配置对应的导入序列化器。"
        data = self.import_serializer_class(queryset, many=True, request=request).data
        if isinstance(self.import_field_dict, MethodType) or isinstance(self.import_field_dict, FunctionType):
            self.import_field_dict = self.import_field_dict()
        # 导出excel 表
        response = HttpResponse(content_type="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet")
        response["Access-Control-Expose-Headers"] = "Content-Disposition"
        response["content-disposition"] = f"attachment;filename={quote(str(f'导出{get_verbose_name(queryset)}.xlsx'))}"
        wb = Workbook()
        ws1 = wb.create_sheet("data", 1)
        ws1.sheet_state = "hidden"
        ws = wb.active
        header_data = ["序号", "更新主键(勿改)"]
        hidden_header = ["#", "id"]
        # ----设置选项----
        validation_data_dict = {}
        for index, item in enumerate(self.import_field_dict.items()):
            items = list(item)
            key = items[0]
            value = items[1]
            if isinstance(value, dict):
                header_data.append(value.get("title"))
                hidden_header.append(value.get("display"))
                choices = value.get("choices", {})
                if choices.get("data"):
                    data_list = []
                    data_list.extend(choices.get("data").keys())
                    validation_data_dict[value.get("title")] = data_list
                elif choices.get("queryset") and choices.get("values_name"):
                    data_list = choices.get("queryset").values_list(choices.get("values_name"), flat=True)
                    validation_data_dict[value.get("title")] = list(data_list)
                else:
                    continue
                column_letter = get_column_letter(len(validation_data_dict))
                dv = DataValidation(
                    type="list",
                    formula1=f"{quote_sheetname('data')}!${column_letter}$2:${column_letter}${len(validation_data_dict[value.get('title')]) + 1}",
                    allow_blank=True,
                )
                ws.add_data_validation(dv)
                dv.add(f"{get_column_letter(index + 3)}2:{get_column_letter(index + 3)}1048576")
            else:
                header_data.append(value)
                hidden_header.append(key)
        # 添加数据列
        ws1.append(list(validation_data_dict.keys()))
        for index, validation_data in enumerate(validation_data_dict.values()):
            for inx, ele in enumerate(validation_data):
                ws1[f"{get_column_letter(index + 1)}{inx + 2}"] = ele
        # --------
        df_len_max = [self.get_string_len(ele) for ele in header_data]
        row = get_column_letter(len(hidden_header) + 1)
        column = 1
        ws.append(header_data)
        for index, results in enumerate(data):
            results_list = []
            for h_index, h_item in enumerate(hidden_header):
                for key, val in results.items():
                    if key == h_item:
                        if val is None or val == "":
                            results_list.append("")
                        elif isinstance(val, list):
                            results_list.append(str(val))
                        else:
                            results_list.append(val)
                        # 计算最大列宽度
                        if isinstance(val, str):
                            result_column_width = self.get_string_len(val)
                            if h_index != 0 and result_column_width > df_len_max[h_index]:
                                df_len_max[h_index] = result_column_width
            ws.append([index + 1, *results_list])
            column += 1
        # 　更新列宽
        for index, width in enumerate(df_len_max):
            ws.column_dimensions[get_column_letter(index + 1)].width = width
        tab = Table(displayName="Table", ref=f"A1:{row}{column}")  # 名称管理器
        style = TableStyleInfo(
            name="TableStyleLight11",
            showFirstColumn=True,
            showLastColumn=True,
            showRowStripes=True,
            showColumnStripes=True,
        )
        tab.tableStyleInfo = style
        ws.add_table(tab)
        wb.save(response)
        return response


class ExportSerializerMixin:
    """
    自定义导出功能
    """

    # 导出字段
    export_field_label = []
    # 导出序列化器
    export_serializer_class = None
    # 表格表头最大宽度，默认50个字符
    export_column_width = 50

    def is_number(self, num):
        try:
            float(num)
            return True
        except ValueError:
            pass

        try:
            import unicodedata

            unicodedata.numeric(num)
            return True
        except (TypeError, ValueError):
            pass
        return False

    def get_string_len(self, string):
        """
        获取字符串最大长度
        :param string:
        :return:
        """
        length = 4
        if string is None:
            return length
        if self.is_number(string):
            return length
        for char in string:
            length += 2.1 if ord(char) > 256 else 1
        return round(length, 1) if length <= self.export_column_width else self.export_column_width

    @action(methods=["get"], detail=False)
    def export_data(self, request: Request, *args, **kwargs):
        """
        导出功能
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        queryset = self.filter_queryset(self.get_queryset())
        assert self.export_field_label, f"'{self.__class__.__name__}' 请配置对应的导出模板字段。"
        assert self.export_serializer_class, f"'{self.__class__.__name__}' 请配置对应的导出序列化器。"
        data = self.export_serializer_class(queryset, many=True, request=request).data
        logger.info("导出 model=%s 条数=%d", queryset.model._meta.label, len(data))
        # 导出excel 表
        response = HttpResponse(content_type="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet")
        response["Access-Control-Expose-Headers"] = "Content-Disposition"
        response["content-disposition"] = f"attachment;filename={quote(str(f'导出{get_verbose_name(queryset)}.xlsx'))}"
        wb = Workbook()
        ws = wb.active
        header_data = ["序号", *self.export_field_label.values()]
        hidden_header = ["#", *self.export_field_label.keys()]
        df_len_max = [self.get_string_len(ele) for ele in header_data]
        row = get_column_letter(len(self.export_field_label) + 1)
        column = 1
        ws.append(header_data)
        for index, results in enumerate(data):
            results_list = []
            for h_index, h_item in enumerate(hidden_header):
                for key, val in results.items():
                    if key == h_item:
                        if val is None or val == "":
                            results_list.append("")
                        else:
                            results_list.append(val)
                        # 计算最大列宽度
                        result_column_width = self.get_string_len(val)
                        if h_index != 0 and result_column_width > df_len_max[h_index]:
                            df_len_max[h_index] = result_column_width
            ws.append([index + 1, *results_list])
            column += 1
        # 　更新列宽
        for index, width in enumerate(df_len_max):
            ws.column_dimensions[get_column_letter(index + 1)].width = width
        tab = Table(displayName="Table", ref=f"A1:{row}{column}")  # 名称管理器
        style = TableStyleInfo(
            name="TableStyleLight11",
            showFirstColumn=True,
            showLastColumn=True,
            showRowStripes=True,
            showColumnStripes=True,
        )
        tab.tableStyleInfo = style
        ws.add_table(tab)
        wb.save(response)
        return response



#!/usr/bin/env python
"""
@Project ：django_base_ai
@File    ：xss_cleaned.py
@Author  ：congxing.wang
@Date    ：2026/02/25
@Desc    ：XSS 过滤与 HTML 清理
"""

import html as html_module
import re

import bleach
from bleach.css_sanitizer import CSSSanitizer

from django_base_ai.settings import ALLOWED_ATTRS, ALLOWED_STYLES, ALLOWED_TAGS

# 定义危险字符串和事件处理器列表
DANGEROUS_STRINGS = [
    "javascript:",
    "vbscript:",
    "data:",
    "onclick",
    "onerror",
    "onload",
    "onmouseover",
    "onfocus",
    "onblur",
    "onchange",
    "onsubmit",
    "onreset",
    "onselect",
    "onunload",
    "onkeydown",
    "onkeypress",
    "onkeyup",
    "onmousedown",
    "onmousemove",
    "onmouseout",
    "onmouseup",
    "onabort",
    "ondblclick",
    "onresize",
    "onscroll",
    "oncontextmenu",
    "oninput",
    "oninvalid",
    "onsearch",
    "onwheel",
    "oncopy",
    "oncut",
    "onpaste",
    "onbeforecopy",
    "onbeforecut",
    "onbeforepaste",
    "onhashchange",
    "onmessage",
    "onoffline",
    "ononline",
    "onpagehide",
    "onpageshow",
    "onpopstate",
    "onstorage",
    "onunhandledrejection",
    "onbeforeunload",
    "onclose",
    "onlanguagechange",
    "onorientationchange",
    "onvisibilitychange",
    "onbeforeprint",
    "onafterprint",
    "oncanplay",
    "oncanplaythrough",
    "ondurationchange",
    "onemptied",
    "onended",
    "onloadeddata",
    "onloadedmetadata",
    "onloadstart",
    "onpause",
    "onplay",
    "onplaying",
    "onprogress",
    "onratechange",
    "onseeked",
    "onseeking",
    "onstalled",
    "onsuspend",
    "ontimeupdate",
    "onvolumechange",
    "onwaiting",
    "onformchange",
    "onforminput",
    "oninvalid",
    "onsubmit",
    "onreset",
    "script",
    "iframe",
    "object",
    "embed",
    "form",
    "input",
    "textarea",
    "button",
    "select",
    "option",
    "meta",
    "link",
    "base",
    "applet",
    "frameset",
    "frame",
    "noscript",
    "isindex",
    "blink",
    "marquee",
]


def _remove_on_events(html):
    """
    移除所有HTML标签中的on事件属性（包括整个属性及其值）
    支持各种格式：onclick="...", onclick='...', onclick=..., ONCLICK="..."等
    处理所有边界情况，包括换行符、特殊字符、自闭合标签等
    :param html: 需要处理的HTML字符串
    :return: 处理后的HTML字符串
    """
    if not html:
        return html

    # 方法1：在HTML标签内部清理（最准确的方式）
    # 匹配所有HTML标签的开始部分（从<到>之间的内容）
    def clean_tag_attrs(match):
        """清理标签属性中的on事件"""
        tag_start = match.group(1)  # <tag
        tag_content = match.group(2)  # 属性部分
        tag_end = match.group(3)  # > 或 />

        if not tag_content:
            return match.group(0)

        # 移除所有on事件属性及其值
        # 使用更全面的正则表达式，处理各种情况
        cleaned = tag_content

        # 移除带双引号的on事件属性（处理前后可能有空格、换行符的情况）
        # 匹配：onxxx="..." 或 onclick="alert(1)" 等，包括多行内容
        cleaned = re.sub(r'\s*on\w+\s*=\s*"[^"]*"\s*', " ", cleaned, flags=re.IGNORECASE | re.DOTALL)
        # 移除带单引号的on事件属性（包括多行内容）
        cleaned = re.sub(r"\s*on\w+\s*=\s*'[^']*'\s*", " ", cleaned, flags=re.IGNORECASE | re.DOTALL)
        # 移除不带引号的on事件属性（值到下一个空格、>、换行符或引号为止）
        cleaned = re.sub(r'\s*on\w+\s*=\s*[^\s>"\'=]+', " ", cleaned, flags=re.IGNORECASE)

        # 清理多余的空格和换行符
        cleaned = re.sub(r"\s+", " ", cleaned).strip()

        # 重新组装
        if cleaned:
            return f"{tag_start} {cleaned}{tag_end}"
        else:
            return f"{tag_start}{tag_end}"

    # 匹配HTML标签：<tag ... > 或 <tag ... /> 或 <tag ...>
    # 使用非贪婪匹配确保只匹配到标签结束，支持多行属性
    pattern = r"(<[a-zA-Z][a-zA-Z0-9]*)\s*([^>]*?)(/?>)"
    html = re.sub(pattern, clean_tag_attrs, html, flags=re.IGNORECASE | re.DOTALL)

    # 方法2：直接在整个HTML中再次移除on事件属性（双重防护，处理可能遗漏的情况）
    # 处理带双引号的情况：onxxx="..."（包括多行）
    html = re.sub(r'\s+on\w+\s*=\s*"[^"]*"', "", html, flags=re.IGNORECASE | re.DOTALL)
    # 处理带单引号的情况：onxxx='...'（包括多行）
    html = re.sub(r"\s+on\w+\s*=\s*'[^']*'", "", html, flags=re.IGNORECASE | re.DOTALL)
    # 处理不带引号的情况：onxxx=value（值到下一个空格、>、引号或换行符为止）
    html = re.sub(r'\s+on\w+\s*=\s*[^\s>"\'=]+', "", html, flags=re.IGNORECASE)

    # 方法3：处理可能被编码或混淆的on事件（如onclick被拆分成on+click等）
    # 移除任何包含on事件模式的字符串（更激进的清理）
    html = re.sub(r'\s*on\w+\s*[:=]\s*["\'][^"\']*["\']', "", html, flags=re.IGNORECASE)
    html = re.sub(r'\s*on\w+\s*[:=]\s*[^\s>"\'=]+', "", html, flags=re.IGNORECASE)

    return html


def _filter_dangerous_strings(text):
    """
    过滤危险字符串和事件处理器
    :param text: 需要过滤的文本
    :return: 过滤后的文本
    """
    if not text:
        return text

    # 先移除所有on事件属性（更彻底的方式）
    text = _remove_on_events(text)

    # 过滤危险字符串（不区分大小写）
    for dangerous in DANGEROUS_STRINGS:
        # 使用正则表达式匹配，确保匹配完整的单词或协议
        pattern = re.compile(re.escape(dangerous), re.IGNORECASE)
        text = pattern.sub("", text)

    # 过滤 javascript: 和 vbscript: 协议（包括各种编码形式）
    text = re.sub(r"javascript\s*:", "", text, flags=re.IGNORECASE)
    text = re.sub(r"vbscript\s*:", "", text, flags=re.IGNORECASE)
    text = re.sub(r"data\s*:", "", text, flags=re.IGNORECASE)

    # 再次过滤事件处理器属性（on*），作为双重防护
    text = re.sub(r'\s+on\w+\s*=\s*["\'][^"\']*["\']', "", text, flags=re.IGNORECASE)
    text = re.sub(r"\s+on\w+\s*=\s*[^\s>]+", "", text, flags=re.IGNORECASE)

    # 过滤 <script> 标签及其内容
    text = re.sub(r"<script[^>]*>.*?</script>", "", text, flags=re.IGNORECASE | re.DOTALL)
    text = re.sub(r"<script[^>]*>", "", text, flags=re.IGNORECASE)
    text = re.sub(r"</script>", "", text, flags=re.IGNORECASE)

    # 过滤其他危险标签
    dangerous_tags = ["iframe", "object", "embed", "form", "input", "textarea", "button", "select", "option"]
    for tag in dangerous_tags:
        text = re.sub(rf"<{tag}[^>]*>.*?</{tag}>", "", text, flags=re.IGNORECASE | re.DOTALL)
        text = re.sub(rf"<{tag}[^>]*>", "", text, flags=re.IGNORECASE)
        text = re.sub(rf"</{tag}>", "", text, flags=re.IGNORECASE)

    return text


def _preserve_user_mention_spans(html):
    """
    保护用户提及的span标签，这些标签包含特定的属性组合
    如果span标签中包含 style="color: #008AC5;"、userid 和 userinsert="true" 属性，则保留原格式
    :param html: 需要处理的HTML字符串
    :return: (处理后的HTML, 占位符到原始span的映射字典)
    """
    if not html:
        return html, {}

    # 匹配符合条件的span标签
    # 要求包含：style="color: #008AC5;"、userid="xxx"、userinsert="true"
    # 使用正则表达式匹配span标签及其内容，属性顺序可以任意
    # 先匹配所有span标签，然后检查是否包含必需的属性
    def is_valid_user_mention_span(span_tag):
        """检查span标签是否包含必需的属性"""
        # 检查是否包含 style="color: #008AC5;" (允许有或没有分号，不区分大小写)
        # 匹配 style="color: #008AC5;" 或 style="color:#008AC5;" 等变体
        has_style = re.search(r'style\s*=\s*["\'][^"\']*color\s*:\s*#008ac5\s*;?[^"\']*["\']', span_tag, re.IGNORECASE)
        # 检查是否包含 userid 属性（值可以是任意字符串）
        has_userid = re.search(r'userid\s*=\s*["\'][^"\']+["\']', span_tag, re.IGNORECASE)
        # 检查是否包含 userinsert="true"
        has_userinsert = re.search(r'userinsert\s*=\s*["\']true["\']', span_tag, re.IGNORECASE)
        return bool(has_style and has_userid and has_userinsert)

    # 匹配所有span标签（包括内容和闭合标签）
    pattern = r"<span\s+[^>]*>.*?</span>"

    placeholder_map = {}
    counter = 0

    def replace_span(match):
        nonlocal counter
        full_span = match.group(0)
        # 只保护符合条件的span标签
        if is_valid_user_mention_span(full_span):
            # 移除span标签中的on事件属性
            cleaned_span = _remove_on_events(full_span)
            placeholder = f"__PRESERVED_SPAN_{counter}__"
            placeholder_map[placeholder] = cleaned_span
            counter += 1
            return placeholder
        else:
            # 不符合条件的span标签保持原样，后续会被正常处理
            return full_span

    # 替换所有符合条件的span标签为占位符
    processed_html = re.sub(pattern, replace_span, html, flags=re.IGNORECASE | re.DOTALL)

    return processed_html, placeholder_map


def _restore_user_mention_spans(html, placeholder_map):
    """
    恢复被保护的span标签
    :param html: 处理后的HTML字符串
    :param placeholder_map: 占位符到原始span的映射字典
    :return: 恢复后的HTML字符串
    """
    if not placeholder_map:
        return html

    for placeholder, original_span in placeholder_map.items():
        html = html.replace(placeholder, original_span)

    return html


def _preserve_nbsp(html):
    """
    保护 &nbsp; 实体，避免被HTML转义
    匹配 &nbsp; 或 &NBSP; 等大小写变体，但不匹配已经被转义的 &amp;nbsp;
    :param html: 需要处理的HTML字符串
    :return: (处理后的HTML, 占位符到原始&nbsp;的映射字典)
    """
    if not html:
        return html, {}

    placeholder_map = {}
    counter = 0

    # 匹配 &nbsp; (不区分大小写)
    # 使用负向前瞻确保不是 &amp;nbsp; (已经被转义的)
    pattern = r"&(?!amp;)nbsp;"

    def replace_nbsp(match):
        nonlocal counter
        placeholder = f"__PRESERVED_NBSP_{counter}__"
        placeholder_map[placeholder] = "&nbsp;"
        counter += 1
        return placeholder

    # 替换所有 &nbsp; 为占位符（不区分大小写）
    processed_html = re.sub(pattern, replace_nbsp, html, flags=re.IGNORECASE)

    return processed_html, placeholder_map


def _restore_nbsp(html, placeholder_map):
    """
    恢复被保护的 &nbsp; 实体
    :param html: 处理后的HTML字符串
    :param placeholder_map: 占位符到原始&nbsp;的映射字典
    :return: 恢复后的HTML字符串
    """
    if not placeholder_map:
        return html

    for placeholder, original_nbsp in placeholder_map.items():
        html = html.replace(placeholder, original_nbsp)

    return html


def cleaned_string(html="", custome_allowed_styles=[], escape_html=True):
    """
    清理字符串，进行严格的XSS防护。
    对于评论等纯文本场景，应该完全转义HTML。
    特殊处理：保留包含用户提及信息的span标签（style="color: #008AC5;"、userid、userinsert="true"）
    :param html: 需要清理的字符串
    :param custome_allowed_styles: 自定义允许的样式列表
    :param escape_html: 是否对输出进行HTML转义（默认True，适用于纯文本场景）
    :return: 清理后的字符串
    """
    if not html:
        return html

    # 先保护符合条件的用户提及span标签
    html, span_placeholder_map = _preserve_user_mention_spans(html)

    # 保护 &nbsp; 实体，避免被转义
    html, nbsp_placeholder_map = _preserve_nbsp(html)

    # 先过滤危险字符串和事件处理器
    html = _filter_dangerous_strings(html)

    # 如果需要对HTML进行转义（纯文本场景），直接转义并返回
    if escape_html:
        # 对输出进行HTML转义，确保特殊字符被转义
        # 这是最安全的方式，适用于评论等纯文本场景
        cleaned_html = html_module.escape(html)
        # 再次过滤危险字符串（双重防护）
        cleaned_html = _filter_dangerous_strings(cleaned_html)
        # 恢复被保护的 &nbsp; 实体
        cleaned_html = _restore_nbsp(cleaned_html, nbsp_placeholder_map)
        # 恢复被保护的用户提及span标签
        cleaned_html = _restore_user_mention_spans(cleaned_html, span_placeholder_map)
        # 最后再次移除所有on事件，确保完全清除（包括恢复的span标签中的on事件）
        cleaned_html = _remove_on_events(cleaned_html)
        # 再次检查并移除on事件（三重防护，确保完全清除）
        cleaned_html = _remove_on_events(cleaned_html)
        return cleaned_html

    # 如果需要保留某些HTML标签（非纯文本场景），使用bleach进行清理
    # 实例化自定义的 CSS sanitizer
    if len(custome_allowed_styles) > 0:
        css_santizer = CSSSanitizer(allowed_css_properties=custome_allowed_styles)
    else:
        css_santizer = CSSSanitizer(allowed_css_properties=ALLOWED_STYLES)

    # 创建更严格的允许标签列表，确保移除所有危险标签
    safe_tags = [
        tag
        for tag in ALLOWED_TAGS
        if tag
        not in [
            "script",
            "object",
            "embed",
            "form",
            "input",
            "textarea",
            "button",
            "select",
            "option",
            "iframe",
            "meta",
            "link",
            "style",
            "base",
            "applet",
            "frameset",
            "frame",
            "noscript",
        ]
    ]

    # 创建更严格的属性配置，移除所有事件处理器
    safe_attrs = {}
    for tag, attrs in ALLOWED_ATTRS.items():
        # 过滤掉所有以 on 开头的属性（事件处理器）
        filtered_attrs = [attr for attr in attrs if not attr.lower().startswith("on")]
        if filtered_attrs:
            safe_attrs[tag] = filtered_attrs

    # 清理字符串，去除不需要的属性
    cleaned_html = bleach.clean(
        html, tags=safe_tags, attributes=safe_attrs, css_sanitizer=css_santizer, protocols=["http", "https", "mailto"]
    )

    # 再次过滤危险字符串（双重防护）
    cleaned_html = _filter_dangerous_strings(cleaned_html)

    # 恢复被保护的 &nbsp; 实体
    cleaned_html = _restore_nbsp(cleaned_html, nbsp_placeholder_map)
    # 恢复被保护的用户提及span标签
    cleaned_html = _restore_user_mention_spans(cleaned_html, span_placeholder_map)
    # 最后再次移除所有on事件，确保完全清除（包括恢复的span标签中的on事件）
    cleaned_html = _remove_on_events(cleaned_html)
    # 再次检查并移除on事件（三重防护，确保完全清除）
    cleaned_html = _remove_on_events(cleaned_html)

    return cleaned_html


def cleaned_html_file(html="", custome_allowed_styles=[]):
    """
    专门用于清理 HTML 文件的函数，移除所有危险标签和属性
    特殊处理：保留 &nbsp; 实体，避免被转义
    :param html: 需要清理的 HTML 字符串
    :param custome_allowed_styles: 自定义允许的样式列表
    :return: 清理后的字符串
    """
    if not html:
        return html

    # 保护 &nbsp; 实体，避免被转义
    html, nbsp_placeholder_map = _preserve_nbsp(html)

    # 先过滤危险字符串和事件处理器
    html = _filter_dangerous_strings(html)

    # 为 HTML 文件创建更严格的允许标签列表，移除危险标签
    safe_tags = [
        tag
        for tag in ALLOWED_TAGS
        if tag
        not in [
            "script",
            "object",
            "embed",
            "form",
            "input",
            "textarea",
            "button",
            "select",
            "option",
            "iframe",
            "meta",
            "link",
            "style",
            "base",
            "applet",
            "frameset",
            "frame",
            "noscript",
        ]
    ]

    # 为 HTML 文件创建更严格的属性配置，移除事件处理器
    safe_attrs = {
        "*": ["style", "class", "id"],  # 只允许基本的样式和标识属性
        "a": ["href", "title", "target"],  # a 标签允许的属性（过滤掉onclick等）
        "img": ["src", "alt", "width", "height", "title"],  # img 标签允许的属性
        "table": ["border", "cellpadding", "cellspacing", "width"],
        "td": ["colspan", "rowspan", "width", "height"],
        "th": ["colspan", "rowspan", "width", "height"],
    }

    # 过滤掉所有事件处理器属性
    for tag in safe_attrs:
        if isinstance(safe_attrs[tag], list):
            safe_attrs[tag] = [attr for attr in safe_attrs[tag] if not attr.lower().startswith("on")]

    # 实例化自定义的 CSS sanitizer
    if len(custome_allowed_styles) > 0:
        css_santizer = CSSSanitizer(allowed_css_properties=custome_allowed_styles)
    else:
        css_santizer = CSSSanitizer(allowed_css_properties=ALLOWED_STYLES)

    # 清理字符串，使用更严格的配置
    cleaned_html = bleach.clean(
        html, tags=safe_tags, attributes=safe_attrs, css_sanitizer=css_santizer, protocols=["http", "https", "mailto"]
    )

    # 再次过滤危险字符串（双重防护）
    cleaned_html = _filter_dangerous_strings(cleaned_html)

    # 恢复被保护的 &nbsp; 实体
    cleaned_html = _restore_nbsp(cleaned_html, nbsp_placeholder_map)

    # 最后再次移除所有on事件，确保完全清除（包括恢复的内容中的on事件）
    cleaned_html = _remove_on_events(cleaned_html)
    # 再次检查并移除on事件（双重防护，确保完全清除）
    cleaned_html = _remove_on_events(cleaned_html)

    # 返回处理后的 HTML 字符串
    return cleaned_html


if __name__ == "__main__":
    # 测试XSS防护功能
    test_cases = [
        "<script>alert(/XSS/);</script>",
        "<img src=1 onerror=alert(1) />",
        '<a href="javascript:alert(1)">Click</a>',
        '<p onclick="alert(1)">Test</p>',
        '<iframe src="javascript:alert(1)"></iframe>',
        "<p>正常文本</p>",
        "<script>alert(/XSS/;</script>",  # 用户报告的测试用例
        '<div onclick="alert(1)" onmouseover="alert(2)">测试</div>',  # 多个on事件
        "<span onclick='alert(1)'>测试</span>",  # 单引号
        '<p ONCLICK="alert(1)">大小写测试</p>',  # 大小写不敏感
        '<a href="#" onclick="alert(1)" onfocus="alert(2)">链接</a>',  # 多个on事件
        '<img src="test.jpg" onload="alert(1)" onerror="alert(2)" />',  # 自闭合标签
    ]

    print("=" * 60)
    print("XSS防护测试")
    print("=" * 60)

    for i, test_html in enumerate(test_cases, 1):
        print(f"\n测试用例 {i}: {test_html}")
        cleaned = cleaned_string(test_html, escape_html=True)
        print(f"清理后: {cleaned}")
        print(
            f"是否包含危险内容: {'是' if any(d in cleaned.lower() for d in ['script', 'onclick', 'javascript:', 'alert'] if d) else '否'}"
        )

    print("\n" + "=" * 60)
    print("用户提及span标签保留测试")
    print("=" * 60)

    # 测试用户提及span标签的保留功能
    user_mention_test = '34324<span style="color: #008AC5;" userid="134" userinsert="true">@测试用户5(TEST05)</span>&nbsp;234324 <h1>A<h1>内容<span style="color: #008AC5;" userid="134" userinsert="true">@测试用户5(TEST05)</span>&nbsp;内容'
    print(f"\n原始内容: {user_mention_test}")
    cleaned_mention = cleaned_string(user_mention_test, escape_html=True)
    print(f"清理后: {cleaned_mention}")
    # 检查是否保留了用户提及的span标签
    has_preserved_span = (
        '<span style="color: #008AC5;" userid="134" userinsert="true" onclick="alert(1)">' in cleaned_mention
    )
    print(f"是否保留了用户提及span标签: {'是' if has_preserved_span else '否'}")

    # 测试不符合条件的span标签应该被转义
    normal_span_test = '<span class="test">普通span</span>'
    print(f"\n普通span测试: {normal_span_test}")
    cleaned_normal = cleaned_string(normal_span_test, escape_html=True)
    print(f"清理后: {cleaned_normal}")
    # 普通span应该被转义
    is_escaped = "&lt;span" in cleaned_normal or "<span" not in cleaned_normal
    print(f"普通span是否被转义: {'是' if is_escaped else '否'}")

    print("\n" + "=" * 60)
    print("&nbsp; 保留测试")
    print("=" * 60)

    # 测试 &nbsp; 是否被正确保留
    nbsp_test = "测试&nbsp;内容&nbsp;更多内容"
    print(f"\n原始内容: {nbsp_test}")
    cleaned_nbsp = cleaned_string(nbsp_test, escape_html=True)
    print(f"清理后: {cleaned_nbsp}")
    # 检查 &nbsp; 是否被保留（不应该被转义为 &amp;nbsp;）
    has_nbsp = "&nbsp;" in cleaned_nbsp
    has_escaped_nbsp = "&amp;nbsp;" in cleaned_nbsp
    print(f"是否保留了 &nbsp;: {'是' if has_nbsp else '否'}")
    print(f"是否被转义为 &amp;nbsp;: {'是' if has_escaped_nbsp else '否'}")

    # 测试包含 &nbsp; 和用户提及span的混合内容
    mixed_test = '文本&nbsp;<span style="color: #008AC5;" userid="134" userinsert="true" onclick="alert(1)">@用户</span>&nbsp;更多文本'
    print(f"\n混合内容测试: {mixed_test}")
    cleaned_mixed = cleaned_string(mixed_test, escape_html=True)
    print(f"清理后: {cleaned_mixed}")
    has_nbsp_in_mixed = "&nbsp;" in cleaned_mixed
    has_span_in_mixed = '<span style="color: #008AC5;" userid="134" userinsert="true">' in cleaned_mixed
    print(f"是否保留了 &nbsp;: {'是' if has_nbsp_in_mixed else '否'}")
    print(f"是否保留了用户提及span: {'是' if has_span_in_mixed else '否'}")

    print("\n" + "=" * 60)
    print("on事件清除专项测试")
    print("=" * 60)

    # 专门测试on事件清除
    on_event_test_cases = [
        ('<p onclick="alert(1)">测试</p>', "onclick"),
        ('<div onmouseover="alert(2)">测试</div>', "onmouseover"),
        ('<a onclick="alert(1)" onfocus="alert(2)">链接</a>', "onclick"),
        ('<img onload="alert(1)" onerror="alert(2)" />', "onload"),
        ('<span ONCLICK="alert(1)">大小写</span>', "onclick"),
        ("<div onclick='alert(1)'>单引号</div>", "onclick"),
        ("<p onclick=alert(1)>无引号</p>", "onclick"),
        ('<div onclick="alert(1)" class="test">混合属性</div>', "onclick"),
    ]

    for test_html, event_name in on_event_test_cases:
        print(f"\n测试: {test_html}")
        cleaned = cleaned_string(test_html, escape_html=False)
        print(f"清理后: {cleaned}")
        has_event = event_name.lower() in cleaned.lower()
        print(f"是否包含{event_name}事件: {'是' if has_event else '否'}")
        if has_event:
            print("⚠️  警告：仍然包含on事件！")

    print("\n" + "=" * 60)
    print("测试完成")
    print("=" * 60)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
将「大纲」或「frontend-slides 生成的 HTML 演示文稿」转为 .pptx。
支持：1）python-pptx 生成后后处理注入背景色/文字色；2）从 OOXML 模板克隆每页 slide XML 再合并成 pptx，最大限度保留样式。
"""
import html
import logging
import re
import zipfile
from io import BytesIO
from typing import Any

import requests
from pptx import Presentation
from pptx.dml.color import RGBColor
from pptx.util import Inches

logger = logging.getLogger(__name__)

# 默认主题：科技蓝（与 HTML 模板一致）
DEFAULT_BG_HEX = "0C1929"
DEFAULT_TITLE_HEX = "F0F9FF"
DEFAULT_BODY_HEX = "CBD5E1"


def _strip_html(raw: str) -> str:
    """去掉 HTML 标签并解码实体，得到纯文本。"""
    if not raw:
        return ""
    s = re.sub(r"<[^>]+>", " ", raw)
    s = re.sub(r"\s+", " ", s).strip()
    return html.unescape(s)


def parse_html_to_outline(html_content: str) -> list[dict[str, Any]]:
    """
    从 frontend-slides 风格的单文件 HTML 中解析出大纲 [{ title, points }, ...]。
    兼容结构：<section class="slide"> 内含标题（h1/h2）与要点（ul li 或 p）。
    若无法解析则返回空列表。
    """
    if not html_content or not html_content.strip():
        return []
    outline: list[dict[str, Any]] = []
    # 按 section.slide 切分；class 可能为 "slide" / 'slide' / "slide title-slide" 等（不捕获组，避免 split 结果错位）
    section_start = re.compile(
        r"<section[^>]*\bclass\s*=\s*(?:[\"'][^\"']*\bslide\b[^\"']*[\"'])[^>]*>",
        re.IGNORECASE,
    )
    parts = section_start.split(html_content)
    for i, block in enumerate(parts):
        if i == 0:
            continue
        end = block.find("</section>")
        if end != -1:
            block = block[:end]
        title = ""
        points: list[str] = []
        image_url = ""
        # 第一个 h1 或 h2 作为标题
        m = re.search(r"<h[12][^>]*>(.*?)</h[12]>", block, re.DOTALL | re.IGNORECASE)
        if m:
            title = _strip_html(m.group(1))
        # 所有 li 作为要点
        for m in re.finditer(r"<li[^>]*>(.*?)</li>", block, re.DOTALL | re.IGNORECASE):
            pt = _strip_html(m.group(1))
            if pt:
                points.append(pt)
        # 若无 li，可用 p 作为单段“要点”
        if not points:
            for m in re.finditer(r"<p[^>]*>(.*?)</p>", block, re.DOTALL | re.IGNORECASE):
                pt = _strip_html(m.group(1))
                if pt and not pt.startswith("<!--"):
                    points.append(pt)
        # 本页配图：.slide-image img 或任意 img 的 src
        img_m = re.search(r'<img[^>]+src\s*=\s*["\']([^"\']+)["\']', block, re.IGNORECASE)
        if img_m:
            image_url = html.unescape(img_m.group(1).strip())
        outline.append({
            "title": title or f"Slide {len(outline) + 1}",
            "points": points,
            "image_url": image_url,
        })
    return outline


def _fetch_image_bytes(url: str, timeout: int = 15) -> BytesIO | None:
    """从 URL 下载图片并返回 BytesIO，失败返回 None。"""
    if not url or not url.strip().lower().startswith(("http://", "https://")):
        return None
    try:
        resp = requests.get(url.strip(), timeout=timeout, stream=True)
        resp.raise_for_status()
        bio = BytesIO(resp.content)
        bio.seek(0)
        return bio
    except Exception as e:
        logger.warning("Fetch image for pptx failed %s: %s", url[:80], e)
        return None


def outline_to_pptx(outline: list[dict[str, Any]], output_path: str) -> None:
    """
    根据大纲生成 .pptx 并写入 output_path。
    outline: [{ "title": str, "points": [str, ...], "image_url"?: str }, ...]，含 image_url 时在页内插入配图。
    """
    prs = Presentation()
    title_layout = prs.slide_layouts[0]
    content_layout = prs.slide_layouts[1]
    # 配图在幻灯片上的位置与尺寸（右侧，与 HTML 左文右图一致）
    IMG_LEFT = Inches(5.5)
    IMG_TOP = Inches(1.6)
    IMG_WIDTH = Inches(4.0)
    IMG_HEIGHT = Inches(4.2)

    for i, item in enumerate(outline):
        if not isinstance(item, dict):
            continue
        title = (item.get("title") or "").strip() or f"Slide {i + 1}"
        points = item.get("points")
        if not isinstance(points, list):
            points = []
        image_url = (item.get("image_url") or "").strip()

        if i == 0 and not points:
            slide = prs.slides.add_slide(title_layout)
            slide.shapes.title.text = title
            if image_url:
                img_io = _fetch_image_bytes(image_url)
                if img_io:
                    try:
                        slide.shapes.add_picture(img_io, IMG_LEFT, IMG_TOP, IMG_WIDTH, IMG_HEIGHT)
                    except Exception as e:
                        logger.warning("Add picture to title slide failed: %s", e)
            continue

        slide = prs.slides.add_slide(content_layout)
        slide.shapes.title.text = title
        body = slide.shapes.placeholders[1]
        tf = body.text_frame
        tf.clear()
        for j, pt in enumerate(points[:8]):
            text = (str(pt) or "").strip()
            if j == 0:
                tf.paragraphs[0].text = text
            else:
                p = tf.add_paragraph()
                p.text = text
                p.level = 0
        if image_url:
            img_io = _fetch_image_bytes(image_url)
            if img_io:
                try:
                    slide.shapes.add_picture(img_io, IMG_LEFT, IMG_TOP, IMG_WIDTH, IMG_HEIGHT)
                except Exception as e:
                    logger.warning("Add picture to slide %s failed: %s", i + 1, e)

    prs.save(output_path)
    # 后处理：向已生成的 pptx 注入背景色与文字颜色，最大限度保留与 HTML 一致的风格
    _apply_pptx_theme(output_path, DEFAULT_BG_HEX, DEFAULT_TITLE_HEX, DEFAULT_BODY_HEX)


def _apply_pptx_theme(
    pptx_path: str,
    bg_hex: str = DEFAULT_BG_HEX,
    title_hex: str = DEFAULT_TITLE_HEX,
    body_hex: str = DEFAULT_BODY_HEX,
) -> None:
    """
    打开已存在的 .pptx（zip），对每页 slide 的 XML 注入背景色（p:bg）与文字颜色（a:rPr solidFill），
    写回 zip。bg_hex/title_hex/body_hex 为 6 位 RRGGBB（无 #）。用字符串插入避免破坏命名空间。
    """
    bg_hex = bg_hex.upper().replace("#", "")[:6]
    title_hex = title_hex.upper().replace("#", "")[:6]
    body_hex = body_hex.upper().replace("#", "")[:6]
    bg_block = (
        f'<p:bg><p:bgPr><a:solidFill><a:srgbClr val="{bg_hex}"/></a:solidFill></p:bgPr></p:bg>'
    )
    rpr_title = (
        f'<a:rPr><a:solidFill><a:srgbClr val="{title_hex}"/></a:solidFill></a:rPr>'
    )
    rpr_body = (
        f'<a:rPr><a:solidFill><a:srgbClr val="{body_hex}"/></a:solidFill></a:rPr>'
    )

    try:
        with zipfile.ZipFile(pptx_path, "r") as zf:
            slide_names = [n for n in zf.namelist() if n.startswith("ppt/slides/slide") and n.endswith(".xml")]
    except Exception as e:
        logger.warning("Apply pptx theme: open zip failed %s: %s", pptx_path, e)
        return

    modified = {}
    for name in sorted(slide_names):
        try:
            with zipfile.ZipFile(pptx_path, "r") as zf:
                xml_str = zf.read(name).decode("utf-8")
            # 若无 p:bg，在 <p:spTree> 前插入背景块
            if "<p:bg>" not in xml_str and "<p:spTree>" in xml_str:
                xml_str = xml_str.replace("<p:spTree>", bg_block + "\n    <p:spTree>", 1)
            # 为 <a:r> 内无 <a:rPr> 的补上文字颜色（第一个 a:r 用标题色，其余用正文色）
            run_count = [0]
            def add_rpr(m):
                run_count[0] += 1
                full = m.group(0)
                if "<a:rPr>" in full:
                    return full
                rpr = rpr_title if run_count[0] == 1 else rpr_body
                return "<a:r>" + rpr + m.group(1) + "</a:r>"
            xml_str = re.sub(r"<a:r>(.*?)</a:r>", add_rpr, xml_str, flags=re.DOTALL)
            modified[name] = xml_str.encode("utf-8")
        except Exception as e:
            logger.warning("Apply pptx theme slide %s: %s", name, e)

    if not modified:
        return
    try:
        with zipfile.ZipFile(pptx_path, "r") as zf:
            all_names = zf.namelist()
            other_data = {n: zf.read(n) for n in all_names if n not in modified}
        other_data.update(modified)
        with zipfile.ZipFile(pptx_path, "w", zipfile.ZIP_DEFLATED) as zf:
            for n in sorted(other_data.keys()):
                zf.writestr(n, other_data[n])
    except Exception as e:
        logger.warning("Apply pptx theme write back failed: %s", e)


def html_to_pptx(html_content: str, output_path: str) -> list[dict[str, Any]]:
    """
    将 HTML 解析为大纲并生成 .pptx。返回解析得到的大纲；若解析失败则抛出 ValueError。
    """
    outline = parse_html_to_outline(html_content)
    if not outline:
        raise ValueError("未能从 HTML 中解析出幻灯片结构（需包含 section.slide）")
    outline_to_pptx(outline, output_path)
    return outline



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
管理命令：将指定表数据导出为 init_<model_name>.json，供 fixtures 初始化使用。
用法：python manage.py generate_init_json [users|role|dept|menu|api_white_list|dictionary|system_config]，不传则全部导出。
"""
import json
import logging
import os

import django
from django.db.models import QuerySet

from django_base_ai.system.views.system_config import SystemConfigInitSerializer

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "DjangoBaseAi.settings")
django.setup()
from django.core.management.base import BaseCommand
from DjangoBaseAi.settings import BASE_DIR

from django_base_ai.system.models import ApiWhiteList, Dept, Dictionary, Menu, Role, SystemConfig, Users
from django_base_ai.system.views.api_white_list import ApiWhiteListInitSerializer
from django_base_ai.system.views.dept import DeptInitSerializer
from django_base_ai.system.views.dictionary import DictionaryInitSerializer
from django_base_ai.system.views.menu import MenuInitSerializer
from django_base_ai.system.views.role import RoleInitSerializer
from django_base_ai.system.views.user import UsersInitSerializer

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    """
    生产初始化菜单: python3 manage.py generate_init_json 生成初始化的model名
    例如：
    全部生成：python3 manage.py generate_init_json
    只生成某个model的： python3 manage.py generate_init_json users
    """

    def serializer_data(self, serializer, query_set: QuerySet):
        """将 QuerySet 序列化并写入 init_<model_name>.json。"""
        serializer = serializer(query_set, many=True)
        data = json.loads(json.dumps(serializer.data, ensure_ascii=False))
        out_path = os.path.join(BASE_DIR, f"init_{query_set.model._meta.model_name}.json")
        with open(out_path, "w", encoding="utf-8") as f:
            json.dump(data, f, indent=4, ensure_ascii=False)
        logger.info("已导出 %s 条至 %s", query_set.count(), out_path)

    def add_arguments(self, parser):
        parser.add_argument("generate_name", nargs="*", type=str, help="初始化生成的表名")

    def generate_users(self):
        self.serializer_data(UsersInitSerializer, Users.objects.all())

    def generate_role(self):
        self.serializer_data(RoleInitSerializer, Role.objects.all())

    def generate_dept(self):
        self.serializer_data(DeptInitSerializer, Dept.objects.filter(parent_id__isnull=True))

    def generate_menu(self):
        self.serializer_data(MenuInitSerializer, Menu.objects.filter(parent_id__isnull=True))

    def generate_api_white_list(self):
        self.serializer_data(ApiWhiteListInitSerializer, ApiWhiteList.objects.all())

    def generate_dictionary(self):
        self.serializer_data(DictionaryInitSerializer, Dictionary.objects.filter(parent_id__isnull=True))

    def generate_system_config(self):
        self.serializer_data(SystemConfigInitSerializer, SystemConfig.objects.filter(parent_id__isnull=True))

    def handle(self, *args, **options):
        generate_name = options.get("generate_name")
        generate_name_dict = {
            "users": self.generate_users,
            "role": self.generate_role,
            "dept": self.generate_dept,
            "menu": self.generate_menu,
            "api_white_list": self.generate_api_white_list,
            "dictionary": self.generate_dictionary,
            "system_config": self.generate_system_config,
        }
        if not generate_name:
            logger.info("导出全部初始化 JSON")
            for ele in generate_name_dict.keys():
                generate_name_dict[ele]()
            return

        for generate_name in generate_name:
            if generate_name not in generate_name_dict:
                print(f"该初始化方法尚未配置\n{generate_name_dict}")
                raise Exception(f"该初始化方法尚未配置,已配置项:{list(generate_name_dict.keys())}")
            generate_name_dict[generate_name]()
            return


if __name__ == "__main__":
    # with open(os.path.join(BASE_DIR, 'temp_init_menu.json')) as f:
    #     for menu_data in json.load(f):
    #         menu_data['creator'] = 1
    #         menu_data['modifier'] = 1
    #         menu_data['dept_belong_id'] = 1
    #         request.user = Users.objects.order_by('create_datetime').first()
    #         serializer = MenuInitSerializer(data=menu_data, request=request)
    #         serializer.is_valid(raise_exception=True)
    #         serializer.save()
    a = Users.objects.filter()
    print(type(Users.objects.filter()))



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
system 应用 URL 配置：登录/登出/Token、初始化接口、各 ViewSet 路由。
模块加载时执行 dispatch 初始化系统配置与字典（仅首次）。
"""
import logging

from django.urls import path
from rest_framework import routers

from django_base_ai import dispatch

logger = logging.getLogger(__name__)

# ---------- 模块加载时初始化系统配置与字典 ----------
from django_base_ai.system.views.ai_desk_app_manage import AIDeskAppManageViewSet
from django_base_ai.system.views.api_white_list import ApiWhiteListViewSet
from django_base_ai.system.views.apps import AppsViewSet
from django_base_ai.system.views.area import AreaViewSet
from django_base_ai.system.views.base_analyze import AnalyzeView
from django_base_ai.system.views.base_task import BaseTaskViewSet
from django_base_ai.system.views.collaborator import CollaboratorViewSet
from django_base_ai.system.views.datav import DataVViewSet
from django_base_ai.system.views.dept import DeptViewSet
from django_base_ai.system.views.dictionary import DictionaryViewSet, InitDictionaryViewSet
from django_base_ai.system.views.file_list import FileViewSet
from django_base_ai.system.views.folder_list import FolderViewSet
from django_base_ai.system.views.frequently_used_contacts import FrequentlyUsedContactsViewSet
from django_base_ai.system.views.holiday import HolidayView
from django_base_ai.system.views.im_chat_qa import IMChatQAViewSet
from django_base_ai.system.views.im_chat_question import IMChatQuestionViewSet
from django_base_ai.system.views.im_chat_question_group import IMChatQuestionGroupViewSet
from django_base_ai.system.views.im_chat_record_form import IMChatRecordFormViewSet
from django_base_ai.system.views.im_chat_to_group import IMChatToGroupViewSet
from django_base_ai.system.views.im_chat_to_group_message import IMChatToGroupMessageViewSet
from django_base_ai.system.views.im_chat_to_user import IMChatToUserViewSet
from django_base_ai.system.views.im_session import IMSessionViewSet
from django_base_ai.system.views.login import (
    CaptchaView,
    CustomTokenRefreshView,
    LoginView,
    LogoutView,
)
from django_base_ai.system.views.login_log import LoginLogViewSet
from django_base_ai.system.views.logins.custome_login import CustomeLoginJWTAPIView
from django_base_ai.system.views.logins.login import LoginJWTAPIView
from django_base_ai.system.views.menu import MenuViewSet
from django_base_ai.system.views.menu_button import MenuButtonViewSet
from django_base_ai.system.views.message_center import MessageCenterViewSet, websocket_view
from django_base_ai.system.views.message_center_target_user import MessageCenterTargetUserViewSet
from django_base_ai.system.views.my_superiors import MySuperiorsViewSet
from django_base_ai.system.views.operation_log import OperationLogViewSet
from django_base_ai.system.views.role import RoleViewSet
from django_base_ai.system.views.system_config import InitSettingsViewSet, SystemConfigViewSet
from django_base_ai.system.views.tags import TagsViewSet
from django_base_ai.system.views.task_run_history import TaskRunHistoryViewSet
from django_base_ai.system.views.user import UserViewSet

logger.info("开始初始化系统配置与字典")
dispatch.init_system_config()
dispatch.init_dictionary()
logger.info("系统配置与字典初始化完成")

# ---------- ViewSet 路由注册 ----------
# 底座核心：认证、RBAC、组织、配置、日志、文件、消息、任务等（见 help/ARCHITECTURE.md）
# 可选扩展：AI 对话、IM、数据分析等（ai_chat_session、im_*、base_analyze、ai_desk_app_manage）
system_url = routers.SimpleRouter()
# --- 底座：RBAC 与组织 ---
system_url.register(r"menu", MenuViewSet)
system_url.register(r"menu_button", MenuButtonViewSet)
system_url.register(r"role", RoleViewSet)
system_url.register(r"dept", DeptViewSet)
system_url.register(r"user", UserViewSet)
system_url.register(r"operation_log", OperationLogViewSet)
system_url.register(r"dictionary", DictionaryViewSet)
system_url.register(r"area", AreaViewSet)
system_url.register(r"folder", FolderViewSet)
system_url.register(r"file", FileViewSet)
system_url.register(r"api_white_list", ApiWhiteListViewSet)
system_url.register(r"system_config", SystemConfigViewSet)
system_url.register(r"message_center", MessageCenterViewSet)
system_url.register(r"message_center_target_user", MessageCenterTargetUserViewSet)
system_url.register(r"frequently_used_contacts", FrequentlyUsedContactsViewSet)
system_url.register(r"datav", DataVViewSet)
system_url.register(r"task", BaseTaskViewSet)
system_url.register(r"task_run_history", TaskRunHistoryViewSet)
system_url.register(r"apps", AppsViewSet)
system_url.register(r"tags", TagsViewSet)
system_url.register(r"collaborator", CollaboratorViewSet)
system_url.register(r"my_superiors", MySuperiorsViewSet)
# --- 可选扩展：AI 工作台、IM ---
system_url.register(r"ai_desk_app_manage", AIDeskAppManageViewSet)
system_url.register(r"im_session", IMSessionViewSet)
system_url.register(r"im_chat_to_user", IMChatToUserViewSet)
system_url.register(r"im_chat_question_group", IMChatQuestionGroupViewSet)
system_url.register(r"im_chat_question", IMChatQuestionViewSet)
system_url.register(r"im_chat_to_group", IMChatToGroupViewSet)
system_url.register(r"im_chat_to_group_message", IMChatToGroupMessageViewSet)
system_url.register(r"im_chat_qa", IMChatQAViewSet)
system_url.register(r"im_chat_record_form", IMChatRecordFormViewSet)
system_url.register(r"login_log", LoginLogViewSet, basename="login_log")
# --- 可选扩展：数据分析、AI ---
system_url.register(r"base_analyze", AnalyzeView, basename="base_analyze")


# ---------- 非 REST 风格路径（登录、登出、初始化、WebSocket 等）----------
urlpatterns = [
    path("login/", LoginView.as_view(), name="token_obtain_pair"),
    path("sign_login/", LoginJWTAPIView.as_view()),  # 单点登录-第三方集成
    path("custome_login/", CustomeLoginJWTAPIView.as_view()),  # 自定义登录
    path("logout/", LogoutView.as_view(), name="logout"),
    path("token/refresh/", CustomTokenRefreshView.as_view(), name="token_refresh"),
    path("captcha/", CaptchaView.as_view()),
    path("init/dictionary/", InitDictionaryViewSet.as_view()),
    path("init/settings/", InitSettingsViewSet.as_view()),
    path("system_config/save_content/", SystemConfigViewSet.as_view({"put": "save_content"})),
    path("system_config/get_association_table/", SystemConfigViewSet.as_view({"get": "get_association_table"})),
    path("system_config/get_table_data/<int:pk>/", SystemConfigViewSet.as_view({"get": "get_table_data"})),
    path("system_config/get_relation_info/", SystemConfigViewSet.as_view({"get": "get_relation_info"})),
    path("dept_lazy_tree/", DeptViewSet.as_view({"get": "dept_lazy_tree"})),
    path("holiday/", HolidayView.as_view()),  # 节假日
    # path('pdf/', ConvertPdfView.as_view()),  # PDF
    path("ws", websocket_view),
]
urlpatterns += system_url.urls



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

from collections import OrderedDict

from django.core import paginator
from django.core.paginator import InvalidPage
from django.core.paginator import Paginator as DjangoPaginator
from rest_framework.pagination import PageNumberPagination
from rest_framework.response import Response


class CustomPagination(PageNumberPagination):
    page_size = 10
    page_size_query_param = "limit"
    max_page_size = 999
    django_paginator_class = DjangoPaginator

    def paginate_queryset(self, queryset, request, view=None):
        """
        Paginate a queryset if required, either returning a
        page object, or `None` if pagination is not configured for this view.
        """
        empty = True

        page_size = self.get_page_size(request)
        if not page_size:
            return None

        paginator = self.django_paginator_class(queryset, page_size)
        page_number = request.query_params.get(self.page_query_param, 1)
        if page_number in self.last_page_strings:
            page_number = paginator.num_pages

        try:
            self.page = paginator.page(page_number)
        except InvalidPage:
            # msg = self.invalid_page_message.format(
            #     page_number=page_number, message=str(exc)
            # )
            # raise NotFound(msg)
            empty = False
            pass

        if paginator.num_pages > 1 and self.template is not None:
            # The browsable API should display pagination controls.
            self.display_page_controls = True

        self.request = request

        if not empty:
            self.page = []

        return list(self.page)

    def get_paginated_response(self, data):
        code = 2000
        msg = "success"
        page_num = 1
        if self.page and hasattr(self.page, "paginator"):
            page_num = int(self.get_page_number(self.request, self.page.paginator)) or 1
        res = {
            "page": page_num,
            "total": self.page.paginator.count if (self.page and hasattr(self.page, "paginator")) else 0,
            "limit": int(self.get_page_size(self.request)) or 10,
            "is_next": self.page.has_next() if (self.page and hasattr(self.page, "has_next")) else False,
            "is_previous": self.page.has_previous() if (self.page and hasattr(self.page, "has_previous")) else False,
            "data": data,
        }
        if not data:
            code = 2000
            msg = "暂无数据"
            res["data"] = []

        return Response(
            OrderedDict(
                [
                    ("code", code),
                    ("msg", msg),
                    ("data", res),
                ]
            )
        )



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""system 应用配置：Django 启动时 ready() 会加载本应用并执行 signals 等。"""
import logging

from django.apps import AppConfig

logger = logging.getLogger(__name__)


class SystemConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "django_base_ai.system"

    def ready(self):
        """应用就绪时调用，用于注册信号等。"""
        logger.info("正在初始化 django_base_ai.system 应用")
        logger.info("django_base_ai.system 应用初始化完成")



"""AI 会话 AIChatSessionViewSet 接口单元测试。"""

import pytest
from rest_framework import status

from tests.assertions import assert_unauthorized

pytestmark = [pytest.mark.django_db]

API = "/base/api/system/ai_chat_session"


def test_ai_chat_session_list_requires_auth(api_client):
    """GET /ai_chat_session/ 未认证返回 401。"""
    response = api_client.get(API + "/")
    assert_unauthorized(response)


def test_ai_chat_session_list_ok(authenticated_client):
    """GET /ai_chat_session/ 已认证返回 200。"""
    response = authenticated_client.get(API + "/")
    assert response.status_code == status.HTTP_200_OK
    assert response.json().get("code") == 2000


def test_ai_chat_session_chat_action_requires_message(authenticated_client):
    """POST /ai_chat_session/chat/ 无 message 返回 400。"""
    response = authenticated_client.post(API + "/chat/", {}, format="json")
    assert response.status_code == status.HTTP_400_BAD_REQUEST



"""
django_base_ai 包入口。

版本号由 setuptools-scm 在打包时注入，未安装时回退为 0.0.1。
"""

from __future__ import annotations

import pymysql

pymysql.install_as_MySQLdb()  # 提供与 mysqlclient 兼容的 API

__version__: str
try:
    from importlib.metadata import version as _pkg_version

    __version__ = _pkg_version("django_base_ai")
except Exception:  # noqa: S110
    __version__ = "0.0.1"
